@@ -10,73 +10,38 @@
 #include <cstring>
 
 #include "BLI_fileops.h"
+#include "BLI_index_range.hh"
 #include "BLI_listbase.h"
-#include "BLI_path_util.h"
+#include "BLI_path_utils.hh"
 #include "BLI_string.h"
+#include "BLI_task.hh"
 #include "BLI_vector.hh"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
 #include "DNA_image_types.h"
 
 #include "MEM_guardedalloc.h"
 
-#include "IMB_colormanagement.h"
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
-#include "IMB_openexr.h"
+#include "IMB_colormanagement.hh"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
+#include "IMB_openexr.hh"
 
-#include "BKE_colortools.h"
-#include "BKE_global.h"
-#include "BKE_image.h"
-#include "BKE_image_format.h"
-#include "BKE_image_save.h"
-#include "BKE_main.h"
-#include "BKE_report.h"
-#include "BKE_scene.h"
+#include "BKE_colortools.hh"
+#include "BKE_global.hh"
+#include "BKE_image.hh"
+#include "BKE_image_format.hh"
+#include "BKE_image_save.hh"
+#include "BKE_library.hh"
+#include "BKE_main.hh"
+#include "BKE_report.hh"
+#include "BKE_scene.hh"
 
 #include "RE_pipeline.h"
 
 using blender::Vector;
 
-static char imtype_best_depth(ImBuf *ibuf, const char imtype)
-{
-  const char depth_ok = BKE_imtype_valid_depths(imtype);
-
-  if (ibuf->float_buffer.data) {
-    if (depth_ok & R_IMF_CHAN_DEPTH_32) {
-      return R_IMF_CHAN_DEPTH_32;
-    }
-    if (depth_ok & R_IMF_CHAN_DEPTH_24) {
-      return R_IMF_CHAN_DEPTH_24;
-    }
-    if (depth_ok & R_IMF_CHAN_DEPTH_16) {
-      return R_IMF_CHAN_DEPTH_16;
-    }
-    if (depth_ok & R_IMF_CHAN_DEPTH_12) {
-      return R_IMF_CHAN_DEPTH_12;
-    }
-    return R_IMF_CHAN_DEPTH_8;
-  }
-
-  if (depth_ok & R_IMF_CHAN_DEPTH_8) {
-    return R_IMF_CHAN_DEPTH_8;
-  }
-  if (depth_ok & R_IMF_CHAN_DEPTH_12) {
-    return R_IMF_CHAN_DEPTH_12;
-  }
-  if (depth_ok & R_IMF_CHAN_DEPTH_16) {
-    return R_IMF_CHAN_DEPTH_16;
-  }
-  if (depth_ok & R_IMF_CHAN_DEPTH_24) {
-    return R_IMF_CHAN_DEPTH_24;
-  }
-  if (depth_ok & R_IMF_CHAN_DEPTH_32) {
-    return R_IMF_CHAN_DEPTH_32;
-  }
-  return R_IMF_CHAN_DEPTH_8; /* fallback, should not get here */
-}
-
 bool BKE_image_save_options_init(ImageSaveOptions *opts,
                                  Main *bmain,
                                  Scene *scene,
@@ -93,7 +58,7 @@ bool BKE_image_save_options_init(ImageSaveOptions *opts,
     iuser->scene = scene;
   }
 
-  memset(opts, 0, sizeof(*opts));
+  *opts = ImageSaveOptions{};
 
   opts->bmain = bmain;
   opts->scene = scene;
@@ -105,14 +70,11 @@ bool BKE_image_save_options_init(ImageSaveOptions *opts,
   ImBuf *ibuf = BKE_image_acquire_ibuf(ima, iuser, &lock);
 
   if (ibuf) {
-    Scene *scene = opts->scene;
-    bool is_depth_set = false;
     const char *ima_colorspace = ima->colorspace_settings.name;
 
     if (opts->save_as_render) {
       /* Render/compositor output or user chose to save with render settings. */
       BKE_image_format_init_for_write(&opts->im_format, scene, nullptr);
-      is_depth_set = true;
       if (!BKE_image_is_multiview(ima)) {
         /* In case multiview is disabled,
          * render settings would be invalid for render result in this area. */
@@ -123,7 +85,8 @@ bool BKE_image_save_options_init(ImageSaveOptions *opts,
     else {
       BKE_image_format_from_imbuf(&opts->im_format, ibuf);
       if (ima->source == IMA_SRC_GENERATED &&
-          !IMB_colormanagement_space_name_is_data(ima_colorspace)) {
+          !IMB_colormanagement_space_name_is_data(ima_colorspace))
+      {
         ima_colorspace = IMB_colormanagement_role_colorspace_name_get(COLOR_ROLE_DEFAULT_BYTE);
       }
 
@@ -146,6 +109,17 @@ bool BKE_image_save_options_init(ImageSaveOptions *opts,
      * by the image saving code itself. */
     BKE_image_user_file_path_ex(bmain, iuser, ima, opts->filepath, false, false);
 
+    /* For movies, replace extension and add the frame number to avoid writing over the movie file
+     * itself and provide a good default file path. */
+    if (ima->source == IMA_SRC_MOVIE) {
+      char filepath_no_ext[FILE_MAX];
+      STRNCPY(filepath_no_ext, opts->filepath);
+      BLI_path_extension_strip(filepath_no_ext);
+      SNPRINTF(opts->filepath, "%s_%.*d", filepath_no_ext, 4, ibuf->fileframe);
+      BKE_image_path_ext_from_imformat_ensure(
+          opts->filepath, sizeof(opts->filepath), &opts->im_format);
+    }
+
     /* sanitize all settings */
 
     /* unlikely but just in case */
@@ -153,11 +127,6 @@ bool BKE_image_save_options_init(ImageSaveOptions *opts,
       opts->im_format.planes = R_IMF_PLANES_RGBA;
     }
 
-    /* depth, account for float buffer and format support */
-    if (is_depth_set == false) {
-      opts->im_format.depth = imtype_best_depth(ibuf, opts->im_format.imtype);
-    }
-
     /* some formats don't use quality so fallback to scenes quality */
     if (opts->im_format.quality == 0) {
       opts->im_format.quality = scene->r.im_format.quality;
@@ -165,20 +134,21 @@ bool BKE_image_save_options_init(ImageSaveOptions *opts,
 
     /* check for empty path */
     if (guess_path && opts->filepath[0] == 0) {
-      const bool is_prev_save = !STREQ(G.ima, "//");
+      const bool is_prev_save = !STREQ(G.filepath_last_image, "//");
       if (opts->save_as_render) {
         if (is_prev_save) {
-          STRNCPY(opts->filepath, G.ima);
+          STRNCPY(opts->filepath, G.filepath_last_image);
         }
         else {
-          BLI_path_join(opts->filepath, sizeof(opts->filepath), "//", DATA_("untitled"));
+          BLI_path_join(opts->filepath, sizeof(opts->filepath), "//", DATA_("Untitled"));
           BLI_path_abs(opts->filepath, BKE_main_blendfile_path(bmain));
         }
       }
       else {
         BLI_path_join(opts->filepath, sizeof(opts->filepath), "//", ima->id.name + 2);
-        BLI_path_make_safe(opts->filepath);
-        BLI_path_abs(opts->filepath, is_prev_save ? G.ima : BKE_main_blendfile_path(bmain));
+        BLI_path_make_safe_filename(opts->filepath + 2);
+        BLI_path_abs(opts->filepath,
+                     is_prev_save ? G.filepath_last_image : BKE_main_blendfile_path(bmain));
       }
 
       /* append UDIM marker if not present */
@@ -217,25 +187,8 @@ void BKE_image_save_options_update(ImageSaveOptions *opts, const Image *image)
       BKE_color_managed_colorspace_settings_copy(&opts->im_format.linear_colorspace_settings,
                                                  &image->colorspace_settings);
     }
-    else if (opts->im_format.imtype != opts->prev_imtype &&
-             !IMB_colormanagement_space_name_is_data(
-                 opts->im_format.linear_colorspace_settings.name))
-    {
-      const bool linear_float_output = BKE_imtype_requires_linear_float(opts->im_format.imtype);
-
-      /* TODO: detect if the colorspace is linear, not just equal to scene linear. */
-      const bool is_linear = IMB_colormanagement_space_name_is_scene_linear(
-          opts->im_format.linear_colorspace_settings.name);
-
-      /* If changing to a linear float or byte format, ensure we have a compatible color space. */
-      if (linear_float_output && !is_linear) {
-        STRNCPY(opts->im_format.linear_colorspace_settings.name,
-                IMB_colormanagement_role_colorspace_name_get(COLOR_ROLE_DEFAULT_FLOAT));
-      }
-      else if (!linear_float_output && is_linear) {
-        STRNCPY(opts->im_format.linear_colorspace_settings.name,
-                IMB_colormanagement_role_colorspace_name_get(COLOR_ROLE_DEFAULT_BYTE));
-      }
+    else if (opts->im_format.imtype != opts->prev_imtype) {
+      BKE_image_format_update_color_space_for_type(&opts->im_format);
     }
   }
 
@@ -273,7 +226,10 @@ static void image_save_post(ReportList *reports,
                             bool *r_colorspace_changed)
 {
   if (!ok) {
-    BKE_reportf(reports, RPT_ERROR, "Could not write image: %s", strerror(errno));
+    BKE_reportf(reports,
+                RPT_ERROR,
+                "Could not write image: %s",
+                errno ? strerror(errno) : "internal error, see console");
     return;
   }
 
@@ -299,8 +255,8 @@ static void image_save_post(ReportList *reports,
     /* workaround to ensure the render result buffer is no longer used
      * by this image, otherwise can crash when a new render result is
      * created. */
-    imb_freerectImBuf(ibuf);
-    imb_freerectfloatImBuf(ibuf);
+    IMB_free_byte_pixels(ibuf);
+    IMB_free_float_pixels(ibuf);
   }
   if (ELEM(ima->source, IMA_SRC_GENERATED, IMA_SRC_VIEWER)) {
     ima->source = IMA_SRC_FILE;
@@ -389,8 +345,7 @@ static bool image_save_single(ReportList *reports,
 
   /* we need renderresult for exr and rendered multiview */
   rr = BKE_image_acquire_renderresult(opts->scene, ima);
-  const bool is_mono = rr ? BLI_listbase_count_at_most(&rr->views, 2) < 2 :
-                            BLI_listbase_count_at_most(&ima->views, 2) < 2;
+  const bool is_mono = !(rr ? RE_ResultIsMultiView(rr) : BKE_image_is_multiview(ima));
   const bool is_exr_rr = rr && ELEM(imf->imtype, R_IMF_IMTYPE_OPENEXR, R_IMF_IMTYPE_MULTILAYER) &&
                          RE_HasFloatPixels(rr);
   const bool is_multilayer = is_exr_rr && (imf->imtype == R_IMF_IMTYPE_MULTILAYER);
@@ -400,6 +355,7 @@ static bool image_save_single(ReportList *reports,
   if (rr == nullptr) {
     if (imf->imtype == R_IMF_IMTYPE_MULTILAYER) {
       BKE_report(reports, RPT_ERROR, "Did not write, no Multilayer Image");
+      BKE_image_release_renderresult(opts->scene, ima, rr);
       BKE_image_release_ibuf(ima, ibuf, lock);
       return ok;
     }
@@ -412,8 +368,8 @@ static bool image_save_single(ReportList *reports,
                     R"(Did not write, the image doesn't have a "%s" and "%s" views)",
                     STEREO_LEFT_NAME,
                     STEREO_RIGHT_NAME);
+        BKE_image_release_renderresult(opts->scene, ima, rr);
         BKE_image_release_ibuf(ima, ibuf, lock);
-        BKE_image_release_renderresult(opts->scene, ima);
         return ok;
       }
 
@@ -426,19 +382,44 @@ static bool image_save_single(ReportList *reports,
                     R"(Did not write, the image doesn't have a "%s" and "%s" views)",
                     STEREO_LEFT_NAME,
                     STEREO_RIGHT_NAME);
+        BKE_image_release_renderresult(opts->scene, ima, rr);
         BKE_image_release_ibuf(ima, ibuf, lock);
-        BKE_image_release_renderresult(opts->scene, ima);
         return ok;
       }
     }
     BKE_imbuf_stamp_info(rr, ibuf);
   }
 
+  /* Don't write permanently into the render-result. */
+  double rr_ppm_prev[2] = {0, 0};
+
+  if (save_as_render && rr) {
+    /* These could be used in the case of a null `rr`, currently they're not though.
+     * Note that setting zero when there is no `rr` is intentional,
+     * this signifies no valid PPM is set. */
+    double ppm[2] = {0, 0};
+    if (opts->scene) {
+      BKE_scene_ppm_get(&opts->scene->r, ppm);
+    }
+    copy_v2_v2_db(rr_ppm_prev, rr->ppm);
+    copy_v2_v2_db(rr->ppm, ppm);
+  }
+
+  /* From now on, calls to #BKE_image_release_renderresult must restore the PPM beforehand. */
+  auto render_result_restore_ppm = [rr, save_as_render, rr_ppm_prev]() {
+    if (save_as_render && rr) {
+      copy_v2_v2_db(rr->ppm, rr_ppm_prev);
+    }
+  };
+
   /* fancy multiview OpenEXR */
   if (imf->views_format == R_IMF_VIEWS_MULTIVIEW && is_exr_rr) {
     /* save render result */
     ok = BKE_image_render_write_exr(
         reports, rr, opts->filepath, imf, save_as_render, nullptr, layer);
+
+    render_result_restore_ppm();
+    BKE_image_release_renderresult(opts->scene, ima, rr);
     image_save_post(reports, ima, ibuf, ok, opts, true, opts->filepath, r_colorspace_changed);
     BKE_image_release_ibuf(ima, ibuf, lock);
   }
@@ -453,6 +434,9 @@ static bool image_save_single(ReportList *reports,
       ok = BKE_imbuf_write_as(colormanaged_ibuf, opts->filepath, imf, save_copy);
       imbuf_save_post(ibuf, colormanaged_ibuf);
     }
+
+    render_result_restore_ppm();
+    BKE_image_release_renderresult(opts->scene, ima, rr);
     image_save_post(reports,
                     ima,
                     ibuf,
@@ -520,6 +504,9 @@ static bool image_save_single(ReportList *reports,
       ok &= ok_view;
     }
 
+    render_result_restore_ppm();
+    BKE_image_release_renderresult(opts->scene, ima, rr);
+
     if (is_exr_rr) {
       BKE_image_release_ibuf(ima, ibuf, lock);
     }
@@ -529,6 +516,9 @@ static bool image_save_single(ReportList *reports,
     if (imf->imtype == R_IMF_IMTYPE_MULTILAYER) {
       ok = BKE_image_render_write_exr(
           reports, rr, opts->filepath, imf, save_as_render, nullptr, layer);
+
+      render_result_restore_ppm();
+      BKE_image_release_renderresult(opts->scene, ima, rr);
       image_save_post(reports, ima, ibuf, ok, opts, true, opts->filepath, r_colorspace_changed);
       BKE_image_release_ibuf(ima, ibuf, lock);
     }
@@ -593,19 +583,25 @@ static bool image_save_single(ReportList *reports,
         ibuf = IMB_stereo3d_ImBuf(imf, ibuf_stereo[0], ibuf_stereo[1]);
 
         /* save via traditional path */
-        ok = BKE_imbuf_write_as(ibuf, opts->filepath, imf, save_copy);
+        if (ibuf) {
+          ok = BKE_imbuf_write_as(ibuf, opts->filepath, imf, save_copy);
 
-        IMB_freeImBuf(ibuf);
+          IMB_freeImBuf(ibuf);
+        }
       }
 
       for (int i = 0; i < 2; i++) {
         IMB_freeImBuf(ibuf_stereo[i]);
       }
+
+      render_result_restore_ppm();
+      BKE_image_release_renderresult(opts->scene, ima, rr);
     }
   }
-
-  if (rr) {
-    BKE_image_release_renderresult(opts->scene, ima);
+  else {
+    render_result_restore_ppm();
+    BKE_image_release_renderresult(opts->scene, ima, rr);
+    BKE_image_release_ibuf(ima, ibuf, lock);
   }
 
   return ok;
@@ -698,12 +694,156 @@ static float *image_exr_from_scene_linear_to_output(float *rect,
 
   const char *from_colorspace = IMB_colormanagement_role_colorspace_name_get(
       COLOR_ROLE_SCENE_LINEAR);
-  IMB_colormanagement_transform(
+  IMB_colormanagement_transform_float(
       output_rect, width, height, channels, from_colorspace, to_colorspace, false);
 
   return output_rect;
 }
 
+static float *image_exr_from_rgb_to_bw(
+    float *input_buffer, int width, int height, int channels, Vector<float *> &temporary_buffers)
+{
+  float *gray_scale_output = MEM_malloc_arrayN<float>(size_t(width) * size_t(height),
+                                                      "Gray Scale Buffer For EXR");
+  temporary_buffers.append(gray_scale_output);
+
+  blender::threading::parallel_for(
+      blender::IndexRange(height), 1, [&](const blender::IndexRange sub_y_range) {
+        for (const int64_t y : sub_y_range) {
+          for (const int64_t x : blender::IndexRange(width)) {
+            const int64_t index = y * int64_t(width) + x;
+            gray_scale_output[index] = IMB_colormanagement_get_luminance(input_buffer +
+                                                                         index * channels);
+          }
+        }
+      });
+
+  return gray_scale_output;
+}
+
+static float *image_exr_opaque_alpha_buffer(int width,
+                                            int height,
+                                            Vector<float *> &temporary_buffers)
+{
+  float *alpha_output = MEM_malloc_arrayN<float>(size_t(width) * size_t(height),
+                                                 "Opaque Alpha Buffer For EXR");
+  temporary_buffers.append(alpha_output);
+
+  blender::threading::parallel_for(
+      blender::IndexRange(height), 1, [&](const blender::IndexRange sub_y_range) {
+        for (const int64_t y : sub_y_range) {
+          for (const int64_t x : blender::IndexRange(width)) {
+            alpha_output[y * int64_t(width) + x] = 1.0;
+          }
+        }
+      });
+
+  return alpha_output;
+}
+
+static void add_exr_compositing_result(void *exr_handle,
+                                       const RenderResult *render_result,
+                                       const ImageFormatData *imf,
+                                       bool save_as_render,
+                                       const char *view_name,
+                                       int layer,
+                                       Vector<float *> &temporary_buffers)
+{
+  /* Render result has no compositing result. */
+  if (!render_result->have_combined) {
+    return;
+  }
+
+  /* Skip compositing result if we are saving a single layer EXR that is not the compositing
+   * layer, which always has the layer index of 0. */
+  const bool is_multi_layer = !(imf && imf->imtype == R_IMF_IMTYPE_OPENEXR);
+  if (!is_multi_layer && layer != 0) {
+    return;
+  }
+
+  /* Write the compositing result for the view with the given view name, or for all views if no
+   * view name is given. */
+  LISTBASE_FOREACH (RenderView *, render_view, &render_result->views) {
+    if (!render_view->ibuf || !render_view->ibuf->float_buffer.data) {
+      continue;
+    }
+
+    /* If a view name is given, then we skip views that do not match the given view name.
+     * Otherwise, we always add the views. */
+    if (view_name && !STREQ(view_name, render_view->name)) {
+      continue;
+    }
+
+    /* If a view name is given, that means we are writing a single view, so no need to identify the
+     * channel by the view name, and we supply an empty view to the rest of the code. */
+    const char *render_view_name = view_name ? "" : render_view->name;
+
+    /* Compositing results is always a 4-channel RGBA. */
+    const int channels_count_in_buffer = 4;
+    float *output_buffer = (save_as_render) ? image_exr_from_scene_linear_to_output(
+                                                  render_view->ibuf->float_buffer.data,
+                                                  render_result->rectx,
+                                                  render_result->recty,
+                                                  channels_count_in_buffer,
+                                                  imf,
+                                                  temporary_buffers) :
+                                              render_view->ibuf->float_buffer.data;
+
+    /* For multi-layer EXRs, we write the buffer as is with all its 4 channels. */
+    const bool half_float = (imf && imf->depth == R_IMF_CHAN_DEPTH_16);
+    if (is_multi_layer) {
+      for (int i = 0; i < channels_count_in_buffer; i++) {
+        char passname[EXR_PASS_MAXNAME];
+        RE_render_result_full_channel_name(passname, nullptr, "Combined", nullptr, "RGBA", i);
+        IMB_exr_add_channel(exr_handle,
+                            "Composite",
+                            passname,
+                            render_view_name,
+                            channels_count_in_buffer,
+                            channels_count_in_buffer * render_result->rectx,
+                            output_buffer + i,
+                            half_float);
+      }
+      continue;
+    }
+
+    /* For single layer EXR, we only add the channels specified in the image format and do any
+     * needed color format conversion.
+     *
+     * In case of a single required channel, we need to do RGBA to BW conversion. */
+    const int required_channels = imf ? imf->planes / 8 : 4;
+    if (required_channels == 1) {
+      float *gray_scale_output = image_exr_from_rgb_to_bw(output_buffer,
+                                                          render_result->rectx,
+                                                          render_result->recty,
+                                                          channels_count_in_buffer,
+                                                          temporary_buffers);
+      IMB_exr_add_channel(exr_handle,
+                          "",
+                          "V",
+                          render_view_name,
+                          1,
+                          render_result->rectx,
+                          gray_scale_output,
+                          half_float);
+      continue;
+    }
+
+    /* Add RGB[A] channels. This will essentially skip the alpha channel if only three channels
+     * were required. */
+    for (int i = 0; i < required_channels; i++) {
+      IMB_exr_add_channel(exr_handle,
+                          "",
+                          std::string(1, "RGBA"[i]).c_str(),
+                          render_view_name,
+                          channels_count_in_buffer,
+                          channels_count_in_buffer * render_result->rectx,
+                          output_buffer + i,
+                          half_float);
+    }
+  }
+}
+
 bool BKE_image_render_write_exr(ReportList *reports,
                                 const RenderResult *rr,
                                 const char *filepath,
@@ -713,10 +853,7 @@ bool BKE_image_render_write_exr(ReportList *reports,
                                 int layer)
 {
   void *exrhandle = IMB_exr_get_handle();
-  const bool half_float = (imf && imf->depth == R_IMF_CHAN_DEPTH_16);
   const bool multi_layer = !(imf && imf->imtype == R_IMF_IMTYPE_OPENEXR);
-  const int channels = (!multi_layer && imf && imf->planes == R_IMF_PLANES_RGB) ? 3 : 4;
-  Vector<float *> tmp_output_rects;
 
   /* Write first layer if not multilayer and no layer was specified. */
   if (!multi_layer && layer == -1) {
@@ -733,57 +870,12 @@ bool BKE_image_render_write_exr(ReportList *reports,
     }
   }
 
-  /* Compositing result. */
-  if (rr->have_combined) {
-    LISTBASE_FOREACH (RenderView *, rview, &rr->views) {
-      if (!rview->ibuf || !rview->ibuf->float_buffer.data) {
-        continue;
-      }
-
-      const char *viewname = rview->name;
-      if (view) {
-        if (!STREQ(view, viewname)) {
-          continue;
-        }
-
-        viewname = "";
-      }
-
-      /* Skip compositing if only a single other layer is requested. */
-      if (!multi_layer && layer != 0) {
-        continue;
-      }
-
-      float *output_rect =
-          (save_as_render) ?
-              image_exr_from_scene_linear_to_output(
-                  rview->ibuf->float_buffer.data, rr->rectx, rr->recty, 4, imf, tmp_output_rects) :
-              rview->ibuf->float_buffer.data;
-
-      for (int a = 0; a < channels; a++) {
-        char passname[EXR_PASS_MAXNAME];
-        char layname[EXR_PASS_MAXNAME];
-        /* "A" is not used if only "RGB" channels are output. */
-        const char *chan_id = "RGBA";
-
-        if (multi_layer) {
-          RE_render_result_full_channel_name(passname, nullptr, "Combined", nullptr, chan_id, a);
-          STRNCPY(layname, "Composite");
-        }
-        else {
-          passname[0] = chan_id[a];
-          passname[1] = '\0';
-          layname[0] = '\0';
-        }
-
-        IMB_exr_add_channel(
-            exrhandle, layname, passname, viewname, 4, 4 * rr->rectx, output_rect + a, half_float);
-      }
-    }
-  }
+  Vector<float *> tmp_output_rects;
+  add_exr_compositing_result(exrhandle, rr, imf, save_as_render, view, layer, tmp_output_rects);
 
   /* Other render layers. */
   int nr = (rr->have_combined) ? 1 : 0;
+  const bool has_multiple_layers = BLI_listbase_count_at_most(&rr->layers, 2) > 1;
   LISTBASE_FOREACH (RenderLayer *, rl, &rr->layers) {
     /* Skip other render layers if requested. */
     if (!multi_layer && nr != layer) {
@@ -792,14 +884,14 @@ bool BKE_image_render_write_exr(ReportList *reports,
     }
     nr++;
 
-    LISTBASE_FOREACH (RenderPass *, rp, &rl->passes) {
+    LISTBASE_FOREACH (RenderPass *, render_pass, &rl->passes) {
       /* Skip non-RGBA and Z passes if not using multi layer. */
-      if (!multi_layer && !STR_ELEM(rp->name, RE_PASSNAME_COMBINED, "")) {
+      if (!multi_layer && !STR_ELEM(render_pass->name, RE_PASSNAME_COMBINED, "")) {
         continue;
       }
 
       /* Skip pass if it does not match the requested view(s). */
-      const char *viewname = rp->view;
+      const char *viewname = render_pass->view;
       if (view) {
         if (!STREQ(view, viewname)) {
           continue;
@@ -810,42 +902,104 @@ bool BKE_image_render_write_exr(ReportList *reports,
 
       /* We only store RGBA passes as half float, for
        * others precision loss can be problematic. */
-      const bool pass_RGBA = STR_ELEM(rp->chan_id, "RGB", "RGBA", "R", "G", "B", "A");
+      const bool pass_RGBA = RE_RenderPassIsColor(render_pass);
+      const bool half_float = (imf && imf->depth == R_IMF_CHAN_DEPTH_16);
       const bool pass_half_float = half_float && pass_RGBA;
 
       /* Color-space conversion only happens on RGBA passes. */
       float *output_rect = (save_as_render && pass_RGBA) ?
-                               image_exr_from_scene_linear_to_output(rp->ibuf->float_buffer.data,
-                                                                     rr->rectx,
-                                                                     rr->recty,
-                                                                     rp->channels,
-                                                                     imf,
-                                                                     tmp_output_rects) :
-                               rp->ibuf->float_buffer.data;
+                               image_exr_from_scene_linear_to_output(
+                                   render_pass->ibuf->float_buffer.data,
+                                   rr->rectx,
+                                   rr->recty,
+                                   render_pass->channels,
+                                   imf,
+                                   tmp_output_rects) :
+                               render_pass->ibuf->float_buffer.data;
 
-      for (int a = 0; a < std::min(channels, rp->channels); a++) {
-        /* Save Combined as RGBA or RGB if single layer save. */
-        char passname[EXR_PASS_MAXNAME];
-        char layname[EXR_PASS_MAXNAME];
+      /* For multi-layer EXRs, we write the pass as is with all of its channels. */
+      if (multi_layer) {
+        for (int i = 0; i < render_pass->channels; i++) {
+          char passname[EXR_PASS_MAXNAME];
+          char layname[EXR_PASS_MAXNAME];
 
-        if (multi_layer) {
-          RE_render_result_full_channel_name(passname, nullptr, rp->name, nullptr, rp->chan_id, a);
-          STRNCPY(layname, rl->name);
-        }
-        else {
-          passname[0] = rp->chan_id[a];
-          passname[1] = '\0';
-          layname[0] = '\0';
+          /* A single unnamed layer indicates that the pass name should be used as the layer name,
+           * while the pass name should be the channel ID. */
+          if (!has_multiple_layers && rl->name[0] == '\0') {
+            passname[0] = render_pass->chan_id[i];
+            passname[1] = '\0';
+            STRNCPY(layname, render_pass->name);
+          }
+          else {
+            RE_render_result_full_channel_name(
+                passname, nullptr, render_pass->name, nullptr, render_pass->chan_id, i);
+            STRNCPY(layname, rl->name);
+          }
+
+          IMB_exr_add_channel(exrhandle,
+                              layname,
+                              passname,
+                              viewname,
+                              render_pass->channels,
+                              render_pass->channels * rr->rectx,
+                              output_rect + i,
+                              pass_half_float);
         }
+        continue;
+      }
 
-        IMB_exr_add_channel(exrhandle,
-                            layname,
-                            passname,
-                            viewname,
-                            rp->channels,
-                            rp->channels * rr->rectx,
-                            output_rect + a,
-                            pass_half_float);
+      /* For single layer EXR, we only add the channels specified in the image format and do any
+       * needed color format conversion.
+       *
+       * First, if the required channels equal the pass channels, we add the channels as is. Or,
+       * we add the RGB[A] channels if the pass is RGB[A] and we require RGB[A]. If the alpha
+       * channel is required but does not exist in the pass, it will be added below. */
+      const int required_channels = imf ? imf->planes / 8 : 4;
+      if (required_channels == render_pass->channels ||
+          (required_channels != 1 && render_pass->channels != 1))
+      {
+        for (int i = 0; i < std::min(required_channels, render_pass->channels); i++) {
+          IMB_exr_add_channel(exrhandle,
+                              "",
+                              std::string(1, render_pass->chan_id[i]).c_str(),
+                              viewname,
+                              render_pass->channels,
+                              render_pass->channels * rr->rectx,
+                              output_rect + i,
+                              pass_half_float);
+        }
+      }
+      else if (required_channels == 1) {
+        /* In case of a single required channel, we need to do RGB[A] to BW conversion. We know the
+         * input is RGB[A] and not single channel because it filed the condition above. */
+        float *gray_scale_output = image_exr_from_rgb_to_bw(
+            output_rect, rr->rectx, rr->recty, render_pass->channels, tmp_output_rects);
+        IMB_exr_add_channel(
+            exrhandle, "", "V", viewname, 1, rr->rectx, gray_scale_output, pass_half_float);
+      }
+      else if (render_pass->channels == 1) {
+        /* In case of a single channel pass, we need to broadcast the same channel for each of the
+         * RGB channels that are required. We know the RGB is required because single channel
+         * requirement was handled above. The alpha channel will be added later. */
+        for (int i = 0; i < 3; i++) {
+          IMB_exr_add_channel(exrhandle,
+                              "",
+                              std::string(1, "RGB"[i]).c_str(),
+                              viewname,
+                              1,
+                              rr->rectx,
+                              output_rect,
+                              pass_half_float);
+        }
+      }
+
+      /* Add an opaque alpha channel if the pass contains no alpha channel but an alpha channel is
+       * required. */
+      if (required_channels == 4 && render_pass->channels < 4) {
+        float *alpha_output = image_exr_opaque_alpha_buffer(
+            rr->rectx, rr->recty, tmp_output_rects);
+        IMB_exr_add_channel(
+            exrhandle, "", "A", viewname, 1, rr->rectx, alpha_output, pass_half_float);
       }
     }
   }
@@ -854,9 +1008,10 @@ bool BKE_image_render_write_exr(ReportList *reports,
 
   BLI_file_ensure_parent_dir_exists(filepath);
 
-  int compress = (imf ? imf->exr_codec : 0);
+  const int compress = (imf ? imf->exr_codec : 0);
+  const int quality = (imf ? imf->quality : 90);
   bool success = IMB_exr_begin_write(
-      exrhandle, filepath, rr->rectx, rr->recty, compress, rr->stamp_data);
+      exrhandle, filepath, rr->rectx, rr->recty, rr->ppm, compress, quality, rr->stamp_data);
   if (success) {
     IMB_exr_write_channels(exrhandle);
   }
@@ -883,7 +1038,9 @@ static void image_render_print_save_message(ReportList *reports,
 {
   if (ok) {
     /* no need to report, just some helpful console info */
-    printf("Saved: '%s'\n", filepath);
+    if (!G.quiet) {
+      printf("Saved: '%s'\n", filepath);
+    }
   }
   else {
     /* report on error since users will want to know what failed */
@@ -892,15 +1049,15 @@ static void image_render_print_save_message(ReportList *reports,
   }
 }
 
-static int image_render_write_stamp_test(ReportList *reports,
-                                         const Scene *scene,
-                                         const RenderResult *rr,
-                                         ImBuf *ibuf,
-                                         const char *filepath,
-                                         const ImageFormatData *imf,
-                                         const bool stamp)
+static bool image_render_write_stamp_test(ReportList *reports,
+                                          const Scene *scene,
+                                          const RenderResult *rr,
+                                          ImBuf *ibuf,
+                                          const char *filepath,
+                                          const ImageFormatData *imf,
+                                          const bool stamp)
 {
-  int ok;
+  bool ok;
 
   if (stamp) {
     /* writes the name of the individual cameras */
@@ -919,7 +1076,9 @@ bool BKE_image_render_write(ReportList *reports,
                             RenderResult *rr,
                             const Scene *scene,
                             const bool stamp,
-                            const char *filepath_basis)
+                            const char *filepath_basis,
+                            const ImageFormatData *format,
+                            bool save_as_render)
 {
   bool ok = true;
 
@@ -928,16 +1087,22 @@ bool BKE_image_render_write(ReportList *reports,
   }
 
   ImageFormatData image_format;
-  BKE_image_format_init_for_write(&image_format, scene, nullptr);
+  BKE_image_format_init_for_write(&image_format, scene, format);
 
-  const bool is_mono = BLI_listbase_count_at_most(&rr->views, 2) < 2;
+  if (!save_as_render) {
+    BKE_color_managed_colorspace_settings_copy(&image_format.linear_colorspace_settings,
+                                               &format->linear_colorspace_settings);
+  }
+
+  const bool is_mono = !RE_ResultIsMultiView(rr);
   const bool is_exr_rr = ELEM(
                              image_format.imtype, R_IMF_IMTYPE_OPENEXR, R_IMF_IMTYPE_MULTILAYER) &&
                          RE_HasFloatPixels(rr);
   const float dither = scene->r.dither_intensity;
 
   if (image_format.views_format == R_IMF_VIEWS_MULTIVIEW && is_exr_rr) {
-    ok = BKE_image_render_write_exr(reports, rr, filepath_basis, &image_format, true, nullptr, -1);
+    ok = BKE_image_render_write_exr(
+        reports, rr, filepath_basis, &image_format, save_as_render, nullptr, -1);
     image_render_print_save_message(reports, filepath_basis, ok, errno);
   }
 
@@ -955,7 +1120,8 @@ bool BKE_image_render_write(ReportList *reports,
       }
 
       if (is_exr_rr) {
-        ok = BKE_image_render_write_exr(reports, rr, filepath, &image_format, true, rv->name, -1);
+        ok = BKE_image_render_write_exr(
+            reports, rr, filepath, &image_format, save_as_render, rv->name, -1);
         image_render_print_save_message(reports, filepath, ok, errno);
 
         /* optional preview images for exr */
@@ -970,7 +1136,7 @@ bool BKE_image_render_write(ReportList *reports,
 
           ImBuf *ibuf = RE_render_result_rect_to_ibuf(rr, &image_format, dither, view_id);
           ibuf->planes = 24;
-          IMB_colormanagement_imbuf_for_write(ibuf, true, false, &image_format);
+          IMB_colormanagement_imbuf_for_write(ibuf, save_as_render, false, &image_format);
 
           ok = image_render_write_stamp_test(
               reports, scene, rr, ibuf, filepath, &image_format, stamp);
@@ -981,7 +1147,7 @@ bool BKE_image_render_write(ReportList *reports,
       else {
         ImBuf *ibuf = RE_render_result_rect_to_ibuf(rr, &image_format, dither, view_id);
 
-        IMB_colormanagement_imbuf_for_write(ibuf, true, false, &image_format);
+        IMB_colormanagement_imbuf_for_write(ibuf, save_as_render, false, &image_format);
 
         ok = image_render_write_stamp_test(
             reports, scene, rr, ibuf, filepath, &image_format, stamp);
@@ -1008,33 +1174,41 @@ bool BKE_image_render_write(ReportList *reports,
       for (i = 0; i < 2; i++) {
         int view_id = BLI_findstringindex(&rr->views, names[i], offsetof(RenderView, name));
         ibuf_arr[i] = RE_render_result_rect_to_ibuf(rr, &image_format, dither, view_id);
-        IMB_colormanagement_imbuf_for_write(ibuf_arr[i], true, false, &image_format);
+        IMB_colormanagement_imbuf_for_write(ibuf_arr[i], save_as_render, false, &image_format);
       }
 
       ibuf_arr[2] = IMB_stereo3d_ImBuf(&image_format, ibuf_arr[0], ibuf_arr[1]);
 
-      ok = image_render_write_stamp_test(
-          reports, scene, rr, ibuf_arr[2], filepath, &image_format, stamp);
-
-      /* optional preview images for exr */
-      if (ok && is_exr_rr && (image_format.flag & R_IMF_FLAG_PREVIEW_JPG)) {
-        image_format.imtype = R_IMF_IMTYPE_JPEG90;
-        image_format.depth = R_IMF_CHAN_DEPTH_8;
-
-        if (BLI_path_extension_check(filepath, ".exr")) {
-          filepath[strlen(filepath) - 4] = 0;
-        }
-
-        BKE_image_path_ext_from_imformat_ensure(filepath, sizeof(filepath), &image_format);
-        ibuf_arr[2]->planes = 24;
-
+      if (ibuf_arr[2]) {
         ok = image_render_write_stamp_test(
             reports, scene, rr, ibuf_arr[2], filepath, &image_format, stamp);
+
+        /* optional preview images for exr */
+        if (ok && is_exr_rr && (image_format.flag & R_IMF_FLAG_PREVIEW_JPG)) {
+          image_format.imtype = R_IMF_IMTYPE_JPEG90;
+          image_format.depth = R_IMF_CHAN_DEPTH_8;
+
+          if (BLI_path_extension_check(filepath, ".exr")) {
+            filepath[strlen(filepath) - 4] = 0;
+          }
+
+          BKE_image_path_ext_from_imformat_ensure(filepath, sizeof(filepath), &image_format);
+          ibuf_arr[2]->planes = 24;
+
+          ok = image_render_write_stamp_test(
+              reports, scene, rr, ibuf_arr[2], filepath, &image_format, stamp);
+        }
+      }
+      else {
+        BKE_reportf(reports, RPT_ERROR, "Failed to create stereo image buffer");
+        ok = false;
       }
 
       /* imbuf knows which rects are not part of ibuf */
       for (i = 0; i < 3; i++) {
-        IMB_freeImBuf(ibuf_arr[i]);
+        if (ibuf_arr[i]) {
+          IMB_freeImBuf(ibuf_arr[i]);
+        }
       }
     }
   }
