@@ -10,11 +10,9 @@
 
 #include <climits>
 
-#include "DNA_anim_types.h"
 #include "DNA_movieclip_types.h"
 #include "DNA_scene_types.h"
-#include "RNA_access.hh"
-#include "RNA_prototypes.h"
+#include "RNA_prototypes.hh"
 
 #include "BLI_ghash.h"
 #include "BLI_listbase.h"
@@ -23,15 +21,15 @@
 #include "BLI_math_vector.h"
 #include "BLI_sort_utils.h"
 #include "BLI_task.h"
-#include "BLI_utildefines.h"
 
-#include "BKE_fcurve.h"
+#include "BKE_fcurve.hh"
 #include "BKE_movieclip.h"
 #include "BKE_tracking.h"
 
-#include "IMB_colormanagement.h"
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
+#include "IMB_colormanagement.hh"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
+#include "IMB_interp.hh"
 #include "MEM_guardedalloc.h"
 
 /* == Parameterization constants == */
@@ -117,7 +115,7 @@ static void attach_stabilization_baseline_data(StabContext *ctx,
 static void discard_stabilization_baseline_data(void *val)
 {
   if (val != nullptr) {
-    MEM_freeN(val);
+    MEM_freeN(static_cast<TrackStabilizationBase *>(val));
   }
 }
 
@@ -138,7 +136,7 @@ static FCurve *retrieve_track_weight_animation(MovieClip *clip, MovieTrackingTra
   return id_data_find_fcurve(&clip->id, track, &RNA_MovieTrackingTrack, "weight_stab", 0, nullptr);
 }
 
-static float fetch_from_fcurve(FCurve *animationCurve,
+static float fetch_from_fcurve(const FCurve *animationCurve,
                                int framenr,
                                StabContext *ctx,
                                float default_value)
@@ -204,7 +202,7 @@ static void use_values_from_fcurves(StabContext *ctx, bool toggle)
  */
 static StabContext *init_stabilization_working_context(MovieClip *clip)
 {
-  StabContext *ctx = MEM_cnew<StabContext>("2D stabilization animation runtime data");
+  StabContext *ctx = MEM_callocN<StabContext>("2D stabilization animation runtime data");
   ctx->clip = clip;
   ctx->tracking = &clip->tracking;
   ctx->stab = &clip->tracking.stabilization;
@@ -871,7 +869,7 @@ static void init_all_tracks(StabContext *ctx, float aspect)
   LISTBASE_FOREACH (MovieTrackingTrack *, track, &tracking_camera_object->tracks) {
     TrackStabilizationBase *local_data = access_stabilization_baseline_data(ctx, track);
     if (!local_data) {
-      local_data = MEM_cnew<TrackStabilizationBase>("2D stabilization per track baseline data");
+      local_data = MEM_callocN<TrackStabilizationBase>("2D stabilization per track baseline data");
       attach_stabilization_baseline_data(ctx, track, local_data);
     }
     BLI_assert(local_data != nullptr);
@@ -884,7 +882,7 @@ static void init_all_tracks(StabContext *ctx, float aspect)
     return;
   }
 
-  order = MEM_cnew_array<TrackInitOrder>(track_len, "stabilization track order");
+  order = MEM_calloc_arrayN<TrackInitOrder>(track_len, "stabilization track order");
   if (!order) {
     return;
   }
@@ -1286,34 +1284,79 @@ void BKE_tracking_stabilization_data_get(MovieClip *clip,
   discard_stabilization_working_context(ctx);
 }
 
-using interpolation_func = void (*)(const ImBuf *, ImBuf *, float, float, int, int);
-
 struct TrackingStabilizeFrameInterpolationData {
   ImBuf *ibuf;
   ImBuf *tmpibuf;
   float (*mat)[4];
-
-  interpolation_func interpolation;
+  int tracking_filter;
 };
 
 static void tracking_stabilize_frame_interpolation_cb(void *__restrict userdata,
-                                                      const int j,
+                                                      const int y,
                                                       const TaskParallelTLS *__restrict /*tls*/)
 {
+  using namespace blender;
+
   TrackingStabilizeFrameInterpolationData *data =
       static_cast<TrackingStabilizeFrameInterpolationData *>(userdata);
   ImBuf *ibuf = data->ibuf;
   ImBuf *tmpibuf = data->tmpibuf;
   float(*mat)[4] = data->mat;
 
-  interpolation_func interpolation = data->interpolation;
+  float vec[3] = {0.0f, float(y), 0.0f};
+  float rvec[3];
 
-  for (int i = 0; i < tmpibuf->x; i++) {
-    float vec[3] = {float(i), float(j), 0.0f};
-
-    mul_v3_m4v3(vec, mat, vec);
-
-    interpolation(ibuf, tmpibuf, vec[0], vec[1], i, j);
+  if (ibuf->float_buffer.data) {
+    /* Float image. */
+    float4 *dst = reinterpret_cast<float4 *>(tmpibuf->float_buffer.data) + y * tmpibuf->x;
+    if (data->tracking_filter == TRACKING_FILTER_BILINEAR) {
+      for (int x = 0; x < tmpibuf->x; x++, dst++) {
+        vec[0] = float(x);
+        mul_v3_m4v3(rvec, mat, vec);
+        *dst = imbuf::interpolate_bilinear_border_fl(ibuf, rvec[0], rvec[1]);
+      }
+    }
+    else if (data->tracking_filter == TRACKING_FILTER_BICUBIC) {
+      for (int x = 0; x < tmpibuf->x; x++, dst++) {
+        vec[0] = float(x);
+        mul_v3_m4v3(rvec, mat, vec);
+        *dst = imbuf::interpolate_cubic_bspline_fl(ibuf, rvec[0], rvec[1]);
+      }
+    }
+    else {
+      /* Nearest or fall back to nearest. */
+      for (int x = 0; x < tmpibuf->x; x++, dst++) {
+        vec[0] = float(x);
+        mul_v3_m4v3(rvec, mat, vec);
+        *dst = imbuf::interpolate_nearest_border_fl(ibuf, rvec[0], rvec[1]);
+      }
+    }
+  }
+  else if (ibuf->byte_buffer.data) {
+    /* Byte image. */
+    uchar4 *dst = reinterpret_cast<uchar4 *>(tmpibuf->byte_buffer.data) + y * tmpibuf->x;
+    if (data->tracking_filter == TRACKING_FILTER_BILINEAR) {
+      for (int x = 0; x < tmpibuf->x; x++, dst++) {
+        vec[0] = float(x);
+        mul_v3_m4v3(rvec, mat, vec);
+        *dst = imbuf::interpolate_bilinear_border_byte(ibuf, rvec[0], rvec[1]);
+      }
+    }
+    else if (data->tracking_filter == TRACKING_FILTER_BICUBIC) {
+      for (int x = 0; x < tmpibuf->x; x++, dst++) {
+        vec[0] = float(x);
+        mul_v3_m4v3(rvec, mat, vec);
+        *dst = imbuf::interpolate_cubic_bspline_byte(ibuf, rvec[0], rvec[1]);
+      }
+    }
+    else {
+      /* Nearest or fall back to nearest. */
+      for (int x = 0; x < tmpibuf->x; x++, dst++) {
+        vec[0] = float(x);
+        mul_v3_m4v3(rvec, mat, vec);
+        *dst = imbuf::interpolate_nearest_border_byte(ibuf, rvec[0], rvec[1]);
+      }
+    }
   }
 }
 
@@ -1327,8 +1370,6 @@ ImBuf *BKE_tracking_stabilize_frame(
   int width = ibuf->x, height = ibuf->y;
   float pixel_aspect = tracking->camera.pixel_aspect;
   float mat[4][4];
-  int filter = tracking->stabilization.filter;
-  interpolation_func interpolation = nullptr;
   int ibuf_flags;
 
   if (translation) {
@@ -1359,10 +1400,10 @@ ImBuf *BKE_tracking_stabilize_frame(
   /* Allocate frame for stabilization result, copy alpha mode and color-space. */
   ibuf_flags = 0;
   if (ibuf->byte_buffer.data) {
-    ibuf_flags |= IB_rect;
+    ibuf_flags |= IB_byte_data;
   }
   if (ibuf->float_buffer.data) {
-    ibuf_flags |= IB_rectfloat;
+    ibuf_flags |= IB_float_data;
   }
 
   tmpibuf = IMB_allocImBuf(ibuf->x, ibuf->y, ibuf->planes, ibuf_flags);
@@ -1378,25 +1419,11 @@ ImBuf *BKE_tracking_stabilize_frame(
    * thus we need the inverse of the transformation to apply. */
   invert_m4(mat);
 
-  if (filter == TRACKING_FILTER_NEAREST) {
-    interpolation = nearest_interpolation;
-  }
-  else if (filter == TRACKING_FILTER_BILINEAR) {
-    interpolation = bilinear_interpolation;
-  }
-  else if (filter == TRACKING_FILTER_BICUBIC) {
-    interpolation = bicubic_interpolation;
-  }
-  else {
-    /* fallback to default interpolation method */
-    interpolation = nearest_interpolation;
-  }
-
   TrackingStabilizeFrameInterpolationData data = {};
   data.ibuf = ibuf;
   data.tmpibuf = tmpibuf;
   data.mat = mat;
-  data.interpolation = interpolation;
+  data.tracking_filter = tracking->stabilization.filter;
 
   TaskParallelSettings settings;
   BLI_parallel_range_settings_defaults(&settings);
