@@ -13,10 +13,6 @@
 
 #include "BLI_math_base.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 MINLINE float safe_divide(float a, float b);
 MINLINE float safe_modf(float a, float b);
 MINLINE float safe_logf(float a, float base);
@@ -26,10 +22,6 @@ MINLINE float safe_asinf(float a);
 MINLINE float safe_acosf(float a);
 MINLINE float safe_powf(float base, float exponent);
 
-#ifdef __cplusplus
-}
-#endif
-
 #if BLI_MATH_DO_INLINE
-#  include "intern/math_base_safe_inline.c"  // IWYU pragma: export
+#  include "intern/math_base_safe_inline.cc"  // IWYU pragma: export
 #endif
