@@ -11,17 +11,11 @@
  * \brief Generic memory manipulation API.
  */
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include <stddef.h>
 
 /* it may be defined already */
 #ifndef __BLI_UTILDEFINES_H__
 bool BLI_memory_is_zero(const void *arr, size_t size);
 #endif
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif /* __BLI_MEMORY_UTILS_H__ */
