@@ -687,8 +687,7 @@ static void quad_4edge_subdivide(BMesh *bm,
   int numcuts = params->numcuts;
   int i, j, a, b, s = numcuts + 2 /* , totv = numcuts * 4 + 4 */;
 
-  lines = static_cast<BMVert **>(
-      MEM_callocN(sizeof(BMVert *) * (numcuts + 2) * (numcuts + 2), "q_4edge_split"));
+  lines = MEM_calloc_arrayN<BMVert *>(size_t(numcuts + 2) * size_t(numcuts + 2), "q_4edge_split");
   /* build a 2-dimensional array of verts,
    * containing every vert (and all new ones)
    * in the face */
@@ -805,8 +804,7 @@ static void tri_3edge_subdivide(BMesh *bm,
   lines[0] = (BMVert **)stackarr;
   lines[0][0] = verts[numcuts * 2 + 1];
 
-  lines[numcuts + 1] = static_cast<BMVert **>(
-      MEM_callocN(sizeof(void *) * (numcuts + 2), "triangle vert table 2"));
+  lines[numcuts + 1] = MEM_calloc_arrayN<BMVert *>(numcuts + 2, "triangle vert table 2");
   for (i = 0; i < numcuts; i++) {
     lines[numcuts + 1][i + 1] = verts[i];
   }
@@ -814,8 +812,7 @@ static void tri_3edge_subdivide(BMesh *bm,
   lines[numcuts + 1][numcuts + 1] = verts[numcuts];
 
   for (i = 0; i < numcuts; i++) {
-    lines[i + 1] = static_cast<BMVert **>(
-        MEM_callocN(sizeof(void *) * (2 + i), "triangle vert table row"));
+    lines[i + 1] = MEM_calloc_arrayN<BMVert *>(2 + i, "triangle vert table row");
     a = numcuts * 2 + 2 + i;
     b = numcuts + numcuts - i;
     e = connect_smallest_face(bm, verts[a], verts[b], &f_new);
@@ -1310,7 +1307,6 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
       bm, op, op->slots_out, "geom.out", BM_ALL_NOLOOP, ELE_INNER | ELE_SPLIT | SUBD_SPLIT);
 }
 
-/* editmesh-emulating function */
 void BM_mesh_esubdivide(BMesh *bm,
                         const char edge_hflag,
                         const float smooth,
