@@ -19,6 +19,7 @@
 #include "BKE_object_types.hh"
 #include "BKE_screen.hh"
 
+#include "BLI_listbase.h"
 #include "BLI_math_geom.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_vector.h"
@@ -51,8 +52,7 @@ static void edbm_selectmode_ensure(Scene *scene, BMEditMesh *em, short selectmod
 {
   if ((scene->toolsettings->selectmode & selectmode) == 0) {
     scene->toolsettings->selectmode |= selectmode;
-    em->selectmode = scene->toolsettings->selectmode;
-    EDBM_selectmode_set(em);
+    EDBM_selectmode_set(em, scene->toolsettings->selectmode);
   }
 }
 
@@ -83,6 +83,25 @@ static bool edbm_preselect_or_active(bContext *C, const View3D *v3d, Base **r_ba
   wmGizmoMap *gzmap = show_gizmo ? region->runtime->gizmo_map : nullptr;
   wmGizmoGroup *gzgroup = gzmap ? WM_gizmomap_group_find(gzmap, "VIEW3D_GGT_mesh_preselect_elem") :
                                   nullptr;
+
+  if (gzgroup != nullptr) {
+    /* Check the gizmo can draw, if not the state may be stale
+     * or if the gizmo group has never drawn the list may even be empty, see: #141336.
+     *
+     * NOTE(ideasman42): we could also fail with an error in this case,
+     * however that would be quite disruptive, so fallback to the active element. */
+    if (!WM_gizmo_context_check_drawstep(C, WM_GIZMOMAP_DRAWSTEP_3D)) {
+      /* Typically only reached when attempting to use the tool during animation playback. */
+      gzgroup = nullptr;
+    }
+    else if (BLI_listbase_is_empty(&gzgroup->gizmos)) {
+      /* If the gizmo group is drawing it *should* never be empty.
+       * Even so, avoid crashing if it is - investigate if this is ever reached. */
+      BLI_assert(false);
+      gzgroup = nullptr;
+    }
+  }
+
   if (gzgroup != nullptr) {
     wmGizmo *gz = static_cast<wmGizmo *>(gzgroup->gizmos.first);
     ED_view3d_gizmo_mesh_preselect_get_active(C, gz, r_base, r_ele);
@@ -113,9 +132,9 @@ static ViewContext edbm_preselect_or_active_init_viewcontext(bContext *C,
   return vc;
 }
 
-static int edbm_polybuild_transform_at_cursor_invoke(bContext *C,
-                                                     wmOperator * /*op*/,
-                                                     const wmEvent * /*event*/)
+static wmOperatorStatus edbm_polybuild_transform_at_cursor_invoke(bContext *C,
+                                                                  wmOperator * /*op*/,
+                                                                  const wmEvent * /*event*/)
 {
   Base *basact = nullptr;
   BMElem *ele_act = nullptr;
@@ -166,7 +185,7 @@ void MESH_OT_polybuild_transform_at_cursor(wmOperatorType *ot)
   ot->name = "Poly Build Transform at Cursor";
   ot->idname = "MESH_OT_polybuild_transform_at_cursor";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = edbm_polybuild_transform_at_cursor_invoke;
   ot->poll = EDBM_view3d_poll;
 
@@ -174,12 +193,12 @@ void MESH_OT_polybuild_transform_at_cursor(wmOperatorType *ot)
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
   /* to give to transform */
-  Transform_Properties(ot, P_PROPORTIONAL | P_MIRROR_DUMMY);
+  blender::ed::transform::properties_register(ot, P_PROPORTIONAL | P_MIRROR_DUMMY);
 }
 
-static int edbm_polybuild_delete_at_cursor_invoke(bContext *C,
-                                                  wmOperator *op,
-                                                  const wmEvent * /*event*/)
+static wmOperatorStatus edbm_polybuild_delete_at_cursor_invoke(bContext *C,
+                                                               wmOperator *op,
+                                                               const wmEvent * /*event*/)
 {
   bool changed = false;
   Base *basact = nullptr;
@@ -253,7 +272,7 @@ void MESH_OT_polybuild_delete_at_cursor(wmOperatorType *ot)
   ot->name = "Poly Build Delete at Cursor";
   ot->idname = "MESH_OT_polybuild_delete_at_cursor";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = edbm_polybuild_delete_at_cursor_invoke;
   ot->poll = EDBM_view3d_poll;
 
@@ -261,7 +280,7 @@ void MESH_OT_polybuild_delete_at_cursor(wmOperatorType *ot)
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
   /* to give to transform */
-  Transform_Properties(ot, P_PROPORTIONAL | P_MIRROR_DUMMY);
+  blender::ed::transform::properties_register(ot, P_PROPORTIONAL | P_MIRROR_DUMMY);
 }
 
 /** \} */
@@ -270,7 +289,9 @@ void MESH_OT_polybuild_delete_at_cursor(wmOperatorType *ot)
 /** \name Face at Cursor
  * \{ */
 
-static int edbm_polybuild_face_at_cursor_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus edbm_polybuild_face_at_cursor_invoke(bContext *C,
+                                                             wmOperator *op,
+                                                             const wmEvent *event)
 {
   float center[3];
   bool changed = false;
@@ -423,7 +444,7 @@ void MESH_OT_polybuild_face_at_cursor(wmOperatorType *ot)
   ot->name = "Poly Build Face at Cursor";
   ot->idname = "MESH_OT_polybuild_face_at_cursor";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = edbm_polybuild_face_at_cursor_invoke;
   ot->poll = EDBM_view3d_poll;
 
@@ -436,7 +457,7 @@ void MESH_OT_polybuild_face_at_cursor(wmOperatorType *ot)
                   "Create Quads",
                   "Automatically split edges in triangles to maintain quad topology");
   /* to give to transform */
-  Transform_Properties(ot, P_PROPORTIONAL | P_MIRROR_DUMMY);
+  blender::ed::transform::properties_register(ot, P_PROPORTIONAL | P_MIRROR_DUMMY);
 }
 
 /** \} */
@@ -445,9 +466,9 @@ void MESH_OT_polybuild_face_at_cursor(wmOperatorType *ot)
 /** \name Split at Cursor
  * \{ */
 
-static int edbm_polybuild_split_at_cursor_invoke(bContext *C,
-                                                 wmOperator * /*op*/,
-                                                 const wmEvent *event)
+static wmOperatorStatus edbm_polybuild_split_at_cursor_invoke(bContext *C,
+                                                              wmOperator * /*op*/,
+                                                              const wmEvent *event)
 {
   float center[3];
   bool changed = false;
@@ -512,7 +533,7 @@ void MESH_OT_polybuild_split_at_cursor(wmOperatorType *ot)
   ot->name = "Poly Build Split at Cursor";
   ot->idname = "MESH_OT_polybuild_split_at_cursor";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = edbm_polybuild_split_at_cursor_invoke;
   ot->poll = EDBM_view3d_poll;
 
@@ -520,7 +541,7 @@ void MESH_OT_polybuild_split_at_cursor(wmOperatorType *ot)
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
   /* to give to transform */
-  Transform_Properties(ot, P_PROPORTIONAL | P_MIRROR_DUMMY);
+  blender::ed::transform::properties_register(ot, P_PROPORTIONAL | P_MIRROR_DUMMY);
 }
 
 /** \} */
@@ -529,9 +550,9 @@ void MESH_OT_polybuild_split_at_cursor(wmOperatorType *ot)
 /** \name Dissolve at Cursor
  * \{ */
 
-static int edbm_polybuild_dissolve_at_cursor_invoke(bContext *C,
-                                                    wmOperator *op,
-                                                    const wmEvent * /*event*/)
+static wmOperatorStatus edbm_polybuild_dissolve_at_cursor_invoke(bContext *C,
+                                                                 wmOperator *op,
+                                                                 const wmEvent * /*event*/)
 {
   bool changed = false;
 
@@ -548,7 +569,12 @@ static int edbm_polybuild_dissolve_at_cursor_invoke(bContext *C,
     BMEdge *e_act = (BMEdge *)ele_act;
     BMLoop *l_a, *l_b;
     if (BM_edge_loop_pair(e_act, &l_a, &l_b)) {
-      BMFace *f_new = BM_faces_join_pair(bm, l_a, l_b, true);
+      BMFace *f_double;
+      BMFace *f_new = BM_faces_join_pair(bm, l_a, l_b, true, &f_double);
+      /* See #BM_faces_join note on callers asserting when `r_double` is non-null. */
+      BLI_assert_msg(f_double == nullptr,
+                     "Doubled face detected at " AT ". Resulting mesh may be corrupt.");
+
       if (f_new) {
         changed = true;
       }
@@ -603,7 +629,7 @@ void MESH_OT_polybuild_dissolve_at_cursor(wmOperatorType *ot)
   ot->name = "Poly Build Dissolve at Cursor";
   ot->idname = "MESH_OT_polybuild_dissolve_at_cursor";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = edbm_polybuild_dissolve_at_cursor_invoke;
   ot->poll = EDBM_view3d_poll;
 
