@@ -31,6 +31,7 @@
 #include "BKE_context.hh"
 #include "BKE_gpencil_legacy.h"
 #include "BKE_layer.hh"
+#include "BKE_library.hh"
 #include "BKE_object.hh"
 #include "BKE_tracking.h"
 
@@ -50,6 +51,7 @@
 #include "UI_interface.hh"
 #include "WM_api.hh"
 
+#include "ANIM_armature.hh"
 #include "ANIM_bone_collections.hh"
 
 #include "screen_intern.hh"
@@ -96,7 +98,7 @@ const char *screen_context_dir[] = {
     "annotation_data",
     "annotation_data_owner",
     "active_annotation_layer",
-    /* Grease Pencil v3 */
+    /* Grease Pencil */
     "grease_pencil",
     "active_operator",
     "active_action",
@@ -278,7 +280,7 @@ static eContextResult screen_ctx_visible_or_editable_bones_(const bContext *C,
       /* Attention: X-Axis Mirroring is also handled here... */
       LISTBASE_FOREACH (EditBone *, ebone, arm->edbo) {
         /* first and foremost, bone must be visible and selected */
-        if (EBONE_VISIBLE(arm, ebone)) {
+        if (blender::animrig::bone_is_visible_editbone(arm, ebone)) {
           /* Get 'x-axis mirror equivalent' bone if the X-Axis Mirroring option is enabled
            * so that most users of this data don't need to explicitly check for it themselves.
            *
@@ -305,7 +307,7 @@ static eContextResult screen_ctx_visible_or_editable_bones_(const bContext *C,
             /* only include bones if visible */
             CTX_data_list_add(result, &arm->id, &RNA_EditBone, ebone);
 
-            if ((flipbone) && EBONE_VISIBLE(arm, flipbone) == 0) {
+            if ((flipbone) && blender::animrig::bone_is_visible_editbone(arm, flipbone) == 0) {
               CTX_data_list_add(result, &arm->id, &RNA_EditBone, flipbone);
             }
           }
@@ -348,7 +350,9 @@ static eContextResult screen_ctx_selected_bones_(const bContext *C,
       /* Attention: X-Axis Mirroring is also handled here... */
       LISTBASE_FOREACH (EditBone *, ebone, arm->edbo) {
         /* first and foremost, bone must be visible and selected */
-        if (EBONE_VISIBLE(arm, ebone) && (ebone->flag & BONE_SELECTED)) {
+        if (blender::animrig::bone_is_visible_editbone(arm, ebone) &&
+            (ebone->flag & BONE_SELECTED))
+        {
           /* Get 'x-axis mirror equivalent' bone if the X-Axis Mirroring option is enabled
            * so that most users of this data don't need to explicitly check for it themselves.
            *
@@ -673,7 +677,7 @@ static eContextResult screen_ctx_active_sequence_strip(const bContext *C,
 {
   wmWindow *win = CTX_wm_window(C);
   Scene *scene = WM_window_get_active_scene(win);
-  Strip *strip = SEQ_select_active_get(scene);
+  Strip *strip = blender::seq::select_active_get(scene);
   if (strip) {
     CTX_data_pointer_set(result, &scene->id, &RNA_Strip, strip);
     return CTX_RESULT_OK;
@@ -684,7 +688,7 @@ static eContextResult screen_ctx_sequences(const bContext *C, bContextDataResult
 {
   wmWindow *win = CTX_wm_window(C);
   Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = SEQ_editing_get(scene);
+  Editing *ed = blender::seq::editing_get(scene);
   if (ed) {
     LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
       CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
@@ -698,7 +702,7 @@ static eContextResult screen_ctx_selected_sequences(const bContext *C, bContextD
 {
   wmWindow *win = CTX_wm_window(C);
   Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = SEQ_editing_get(scene);
+  Editing *ed = blender::seq::editing_get(scene);
   if (ed) {
     LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
       if (strip->flag & SELECT) {
@@ -715,14 +719,14 @@ static eContextResult screen_ctx_selected_editable_sequences(const bContext *C,
 {
   wmWindow *win = CTX_wm_window(C);
   Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = SEQ_editing_get(scene);
+  Editing *ed = blender::seq::editing_get(scene);
   if (ed == nullptr) {
     return CTX_RESULT_NO_DATA;
   }
 
-  ListBase *channels = SEQ_channels_displayed_get(ed);
+  ListBase *channels = blender::seq::channels_displayed_get(ed);
   LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
-    if (strip->flag & SELECT && !SEQ_transform_is_locked(channels, strip)) {
+    if (strip->flag & SELECT && !blender::seq::transform_is_locked(channels, strip)) {
       CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
     }
   }
@@ -943,7 +947,7 @@ static eContextResult screen_ctx_sel_actions_impl(const bContext *C,
   blender::Set<bAction *> seen_set;
 
   LISTBASE_FOREACH (bAnimListElem *, ale, &anim_data) {
-    /* In dopesheet check selection status of individual items, skipping
+    /* In dope-sheet check selection status of individual items, skipping
      * if not selected or has no selection flag. This is needed so that
      * selecting action or group rows without any channels works. */
     if (check_selected && ANIM_channel_setting_get(&ac, ale, ACHANNEL_SETTING_SELECT) <= 0) {
@@ -1138,7 +1142,7 @@ static eContextResult screen_ctx_active_strip(const bContext *C, bContextDataRes
 {
   wmWindow *win = CTX_wm_window(C);
   Scene *scene = WM_window_get_active_scene(win);
-  Strip *strip = SEQ_select_active_get(scene);
+  Strip *strip = blender::seq::select_active_get(scene);
   if (strip) {
     CTX_data_pointer_set(result, &scene->id, &RNA_Strip, strip);
     return CTX_RESULT_OK;
@@ -1149,7 +1153,7 @@ static eContextResult screen_ctx_strips(const bContext *C, bContextDataResult *r
 {
   wmWindow *win = CTX_wm_window(C);
   Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = SEQ_editing_get(scene);
+  Editing *ed = blender::seq::editing_get(scene);
   if (ed) {
     LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
       CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
@@ -1163,7 +1167,7 @@ static eContextResult screen_ctx_selected_strips(const bContext *C, bContextData
 {
   wmWindow *win = CTX_wm_window(C);
   Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = SEQ_editing_get(scene);
+  Editing *ed = blender::seq::editing_get(scene);
   if (ed) {
     LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
       if (strip->flag & SELECT) {
@@ -1180,14 +1184,14 @@ static eContextResult screen_ctx_selected_editable_strips(const bContext *C,
 {
   wmWindow *win = CTX_wm_window(C);
   Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = SEQ_editing_get(scene);
+  Editing *ed = blender::seq::editing_get(scene);
   if (ed == nullptr) {
     return CTX_RESULT_NO_DATA;
   }
 
-  ListBase *channels = SEQ_channels_displayed_get(ed);
+  ListBase *channels = blender::seq::channels_displayed_get(ed);
   LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
-    if (strip->flag & SELECT && !SEQ_transform_is_locked(channels, strip)) {
+    if (strip->flag & SELECT && !blender::seq::transform_is_locked(channels, strip)) {
       CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
     }
   }
