@@ -10,10 +10,12 @@
 
 #include "GHOST_C-api.h"
 
+#include "BLI_path_utils.hh"
 #include "BLI_threads.h"
 
 #include "CLG_log.h"
 
+#include "GPU_capabilities.hh"
 #include "gpu_capabilities_private.hh"
 #include "gpu_platform_private.hh"
 
@@ -89,6 +91,25 @@ bool GPU_vulkan_is_supported_driver(VkPhysicalDevice vk_physical_device)
   }
 #endif
 
+#ifdef _WIN32
+  if (vk_physical_device_driver_properties.driverID == VK_DRIVER_ID_QUALCOMM_PROPRIETARY) {
+    /* Any Qualcomm driver older than 31.0.112.0 will not be capable of running blender due
+     * to an issue in their semaphore timeline implementation. The driver could return
+     * timelines that have not been provided by Blender. As Blender uses timelines for resource
+     * management this resulted in resources to be destroyed, that are still in use. */
+
+    /* Public version 31.0.112 uses vulkan driver version 512.827.0. */
+    const uint32_t driver_version = vk_physical_device_properties.properties.driverVersion;
+    constexpr uint32_t version_31_0_112 = VK_MAKE_VERSION(512, 827, 0);
+    if (driver_version < version_31_0_112) {
+      CLOG_WARN(&LOG,
+                "Detected qualcomm driver is not supported. To run the Vulkan backend "
+                "driver 31.0.112.0 or later is required. Switching to OpenGL.");
+      return false;
+    }
+  }
+#endif
+
   return true;
 }
 
@@ -135,6 +156,9 @@ static Vector<StringRefNull> missing_capabilities_get(VkPhysicalDevice vk_physic
   if (features_12.timelineSemaphore == VK_FALSE) {
     missing_capabilities.append("timeline semaphores");
   }
+  if (features_12.bufferDeviceAddress == VK_FALSE) {
+    missing_capabilities.append("buffer device address");
+  }
 
   /* Check device extensions. */
   uint32_t vk_extension_count;
@@ -165,6 +189,25 @@ bool VKBackend::is_supported()
 {
   CLG_logref_init(&LOG);
 
+  /*
+   * Disable implicit layers and only allow layers that we trust.
+   *
+   * Render doc layer is hidden behind a debug flag. There are malicious layers that impersonate
+   * RenderDoc and can crash when loaded. See #139543
+   */
+  std::stringstream allowed_layers;
+  allowed_layers << "VK_LAYER_KHRONOS_*";
+  allowed_layers << ",VK_LAYER_AMD_*";
+  allowed_layers << ",VK_LAYER_INTEL_*";
+  allowed_layers << ",VK_LAYER_NV_*";
+  allowed_layers << ",VK_LAYER_MESA_*";
+  if (bool(G.debug & G_DEBUG_GPU)) {
+    allowed_layers << ",VK_LAYER_LUNARG_*";
+    allowed_layers << ",VK_LAYER_RENDERDOC_*";
+  }
+  BLI_setenv("VK_LOADER_LAYERS_DISABLE", "~implicit~");
+  BLI_setenv("VK_LOADER_LAYERS_ALLOW", allowed_layers.str().c_str());
+
   /* Initialize an vulkan 1.2 instance. */
   VkApplicationInfo vk_application_info = {VK_STRUCTURE_TYPE_APPLICATION_INFO};
   vk_application_info.pApplicationName = "Blender";
@@ -183,7 +226,7 @@ bool VKBackend::is_supported()
     return false;
   }
 
-  // go over all the devices
+  /* Go over all the devices. */
   uint32_t physical_devices_count = 0;
   vkEnumeratePhysicalDevices(vk_instance, &physical_devices_count, nullptr);
   Array<VkPhysicalDevice> vk_physical_devices(physical_devices_count);
@@ -257,26 +300,18 @@ void VKBackend::platform_init()
            "",
            "",
            GPU_ARCHITECTURE_IMR);
+}
 
-  /* Query for all compatible devices */
-  VkApplicationInfo vk_application_info = {VK_STRUCTURE_TYPE_APPLICATION_INFO};
-  vk_application_info.pApplicationName = "Blender";
-  vk_application_info.applicationVersion = VK_MAKE_VERSION(1, 0, 0);
-  vk_application_info.pEngineName = "Blender";
-  vk_application_info.engineVersion = VK_MAKE_VERSION(1, 0, 0);
-  vk_application_info.apiVersion = VK_API_VERSION_1_2;
-
-  VkInstanceCreateInfo vk_instance_info = {VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO};
-  vk_instance_info.pApplicationInfo = &vk_application_info;
-
-  VkInstance vk_instance = VK_NULL_HANDLE;
-  vkCreateInstance(&vk_instance_info, nullptr, &vk_instance);
-  BLI_assert(vk_instance != VK_NULL_HANDLE);
+static void init_device_list(GHOST_ContextHandle ghost_context)
+{
+  GHOST_VulkanHandles vulkan_handles = {};
+  GHOST_GetVulkanHandles(ghost_context, &vulkan_handles);
 
   uint32_t physical_devices_count = 0;
-  vkEnumeratePhysicalDevices(vk_instance, &physical_devices_count, nullptr);
+  vkEnumeratePhysicalDevices(vulkan_handles.instance, &physical_devices_count, nullptr);
   Array<VkPhysicalDevice> vk_physical_devices(physical_devices_count);
-  vkEnumeratePhysicalDevices(vk_instance, &physical_devices_count, vk_physical_devices.data());
+  vkEnumeratePhysicalDevices(
+      vulkan_handles.instance, &physical_devices_count, vk_physical_devices.data());
   int index = 0;
   for (VkPhysicalDevice vk_physical_device : vk_physical_devices) {
     if (missing_capabilities_get(vk_physical_device).is_empty() &&
@@ -295,7 +330,7 @@ void VKBackend::platform_init()
     }
     index++;
   }
-  vkDestroyInstance(vk_instance, nullptr);
+
   std::sort(GPG.devices.begin(), GPG.devices.end(), [&](const GPUDevice &a, const GPUDevice &b) {
     if (a.name == b.name) {
       return a.index < b.index;
@@ -331,6 +366,19 @@ void VKBackend::platform_init(const VKDevice &device)
            GPU_ARCHITECTURE_IMR);
   GPG.devices = devices;
 
+  const VkPhysicalDeviceIDProperties &id_properties = device.physical_device_id_properties_get();
+
+  GPG.device_uuid = Array<uint8_t, 16>(Span<uint8_t>(id_properties.deviceUUID, VK_UUID_SIZE));
+
+  if (id_properties.deviceLUIDValid) {
+    GPG.device_luid = Array<uint8_t, 8>(Span<uint8_t>(id_properties.deviceUUID, VK_LUID_SIZE));
+    GPG.device_luid_node_mask = id_properties.deviceNodeMask;
+  }
+  else {
+    GPG.device_luid.reinitialize(0);
+    GPG.device_luid_node_mask = 0;
+  }
+
   CLOG_INFO(&LOG,
             0,
             "Using vendor [%s] device [%s] driver version [%s].",
@@ -342,6 +390,7 @@ void VKBackend::platform_init(const VKDevice &device)
 void VKBackend::detect_workarounds(VKDevice &device)
 {
   VKWorkarounds workarounds;
+  VKExtensions extensions;
 
   if (G.debug & G_DEBUG_GPU_FORCE_WORKAROUNDS) {
     printf("\n");
@@ -349,35 +398,77 @@ void VKBackend::detect_workarounds(VKDevice &device)
     printf("    Vendor: %s\n", device.vendor_name().c_str());
     printf("    Device: %s\n", device.physical_device_properties_get().deviceName);
     printf("    Driver: %s\n", device.driver_version().c_str());
-    /* Force workarounds. */
+    /* Force workarounds and disable extensions. */
     workarounds.not_aligned_pixel_formats = true;
-    workarounds.shader_output_layer = true;
-    workarounds.shader_output_viewport_index = true;
     workarounds.vertex_formats.r8g8b8 = true;
-    workarounds.fragment_shader_barycentric = true;
-    workarounds.dynamic_rendering = true;
-    workarounds.dynamic_rendering_local_read = true;
-    workarounds.dynamic_rendering_unused_attachments = true;
+    extensions.shader_output_layer = false;
+    extensions.shader_output_viewport_index = false;
+    extensions.fragment_shader_barycentric = false;
+    extensions.dynamic_rendering = false;
+    extensions.dynamic_rendering_local_read = false;
+    extensions.dynamic_rendering_unused_attachments = false;
+    extensions.descriptor_buffer = false;
+    extensions.pageable_device_local_memory = false;
 
     GCaps.render_pass_workaround = true;
 
     device.workarounds_ = workarounds;
+    device.extensions_ = extensions;
     return;
   }
 
-  workarounds.shader_output_layer =
-      !device.physical_device_vulkan_12_features_get().shaderOutputLayer;
-  workarounds.shader_output_viewport_index =
-      !device.physical_device_vulkan_12_features_get().shaderOutputViewportIndex;
-  workarounds.fragment_shader_barycentric = !device.supports_extension(
+  extensions.shader_output_layer =
+      device.physical_device_vulkan_12_features_get().shaderOutputLayer;
+  extensions.shader_output_viewport_index =
+      device.physical_device_vulkan_12_features_get().shaderOutputViewportIndex;
+  extensions.fragment_shader_barycentric = device.supports_extension(
       VK_KHR_FRAGMENT_SHADER_BARYCENTRIC_EXTENSION_NAME);
-  workarounds.dynamic_rendering = !device.supports_extension(
+  extensions.dynamic_rendering = device.supports_extension(
       VK_KHR_DYNAMIC_RENDERING_EXTENSION_NAME);
-  workarounds.dynamic_rendering_local_read = !device.supports_extension(
+  extensions.dynamic_rendering_local_read = device.supports_extension(
       VK_KHR_DYNAMIC_RENDERING_LOCAL_READ_EXTENSION_NAME);
-  workarounds.dynamic_rendering_unused_attachments = !device.supports_extension(
+  extensions.dynamic_rendering_unused_attachments = device.supports_extension(
       VK_EXT_DYNAMIC_RENDERING_UNUSED_ATTACHMENTS_EXTENSION_NAME);
-  workarounds.logic_ops = !device.physical_device_features_get().logicOp;
+  extensions.logic_ops = device.physical_device_features_get().logicOp;
+  /* For stability reasons descriptor buffers have been disabled. */
+#if 0
+  extensions.descriptor_buffer = device.supports_extension(
+      VK_EXT_DESCRIPTOR_BUFFER_EXTENSION_NAME);
+#endif
+  extensions.memory_priority = device.supports_extension(VK_EXT_MEMORY_PRIORITY_EXTENSION_NAME);
+  extensions.pageable_device_local_memory = device.supports_extension(
+      VK_EXT_PAGEABLE_DEVICE_LOCAL_MEMORY_EXTENSION_NAME);
+#ifdef _WIN32
+  extensions.external_memory = device.supports_extension(
+      VK_KHR_EXTERNAL_MEMORY_WIN32_EXTENSION_NAME);
+#elif not defined(__APPLE__)
+  extensions.external_memory = device.supports_extension(VK_KHR_EXTERNAL_MEMORY_FD_EXTENSION_NAME);
+#else
+  extensions.external_memory = false;
+#endif
+
+  /* Descriptor buffers are disabled on the NVIDIA platform due to performance regressions. Both
+   * still seem to be faster than OpenGL.
+   *
+   * See #140125
+   */
+  if (device.vk_physical_device_driver_properties_.driverID == VK_DRIVER_ID_NVIDIA_PROPRIETARY) {
+    extensions.descriptor_buffer = false;
+  }
+
+  /* Running render tests fails consistenly in some scenes. The cause is that too many descriptor
+   * sets are required for rendering resulting in failing allocations of the descriptor buffer. We
+   * work around this issue by not using descriptor buffers on these platforms.
+   *
+   * TODO: recheck when the backed memory gets freed and how to improve it.
+   *
+   * See #141476
+   */
+  if (device.vk_physical_device_driver_properties_.driverID ==
+      VK_DRIVER_ID_INTEL_PROPRIETARY_WINDOWS)
+  {
+    extensions.descriptor_buffer = false;
+  }
 
   /* AMD GPUs don't support texture formats that use are aligned to 24 or 48 bits. */
   if (GPU_type_matches(GPU_DEVICE_ATI, GPU_OS_ANY, GPU_DRIVER_ANY) ||
@@ -399,7 +490,7 @@ void VKBackend::detect_workarounds(VKDevice &device)
   if ((G.debug & G_DEBUG_GPU_FORCE_VULKAN_LOCAL_READ) == 0 &&
       !GPU_type_matches(GPU_DEVICE_QUALCOMM, GPU_OS_ANY, GPU_DRIVER_ANY))
   {
-    workarounds.dynamic_rendering_local_read = true;
+    extensions.dynamic_rendering_local_read = false;
   }
 
   VkFormatProperties format_properties = {};
@@ -408,13 +499,7 @@ void VKBackend::detect_workarounds(VKDevice &device)
   workarounds.vertex_formats.r8g8b8 = (format_properties.bufferFeatures &
                                        VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT) == 0;
 
-  workarounds.fragment_shader_barycentric = !device.supports_extension(
-      VK_KHR_FRAGMENT_SHADER_BARYCENTRIC_EXTENSION_NAME);
-
-  GCaps.render_pass_workaround = workarounds.dynamic_rendering = !device.supports_extension(
-      VK_KHR_DYNAMIC_RENDERING_EXTENSION_NAME);
-  workarounds.dynamic_rendering_unused_attachments = !device.supports_extension(
-      VK_EXT_DYNAMIC_RENDERING_UNUSED_ATTACHMENTS_EXTENSION_NAME);
+  GCaps.render_pass_workaround = !extensions.dynamic_rendering;
 
 #ifdef __APPLE__
   /* Due to a limitation in MoltenVK, attachments should be sequential even when using
@@ -426,6 +511,7 @@ void VKBackend::detect_workarounds(VKDevice &device)
 #endif
 
   device.workarounds_ = workarounds;
+  device.extensions_ = extensions;
 }
 
 void VKBackend::platform_exit()
@@ -437,7 +523,16 @@ void VKBackend::platform_exit()
   }
 }
 
-void VKBackend::delete_resources() {}
+void VKBackend::init_resources()
+{
+  compiler_ = MEM_new<ShaderCompiler>(
+      __func__, GPU_max_parallel_compilations(), GPUWorker::ContextType::Main);
+}
+
+void VKBackend::delete_resources()
+{
+  MEM_delete(compiler_);
+}
 
 void VKBackend::samplers_update()
 {
@@ -482,13 +577,18 @@ Context *VKBackend::context_alloc(void *ghost_window, void *ghost_context)
   BLI_assert(ghost_context != nullptr);
   if (!device.is_initialized()) {
     device.init(ghost_context);
+    device.extensions_get().log();
+    init_device_list((GHOST_ContextHandle)ghost_context);
   }
 
   VKContext *context = new VKContext(ghost_window, ghost_context);
   device.context_register(*context);
   GHOST_SetVulkanSwapBuffersCallbacks((GHOST_ContextHandle)ghost_context,
                                       VKContext::swap_buffers_pre_callback,
-                                      VKContext::swap_buffers_post_callback);
+                                      VKContext::swap_buffers_post_callback,
+                                      VKContext::openxr_acquire_framebuffer_image_callback,
+                                      VKContext::openxr_release_framebuffer_image_callback);
+
   return context;
 }
 
@@ -560,18 +660,35 @@ void VKBackend::render_end()
   thread_data.rendering_depth -= 1;
   BLI_assert_msg(thread_data.rendering_depth >= 0, "Unbalanced `GPU_render_begin/end`");
   if (G.background) {
-    /* Garbage collection when performing background rendering. */
     if (thread_data.rendering_depth == 0) {
       VKContext *context = VKContext::get();
       if (context != nullptr) {
-        context->flush_render_graph(RenderGraphFlushFlags::RENEW_RENDER_GRAPH);
+        context->flush();
       }
-      device.orphaned_data.destroy_discarded_resources(device);
+      std::scoped_lock lock(device.orphaned_data.mutex_get());
+      device.orphaned_data.move_data(device.orphaned_data_render,
+                                     device.orphaned_data.timeline_ + 1);
     }
   }
+
+  /* When performing animation render we want to release any discarded resources during rendering
+   * after each frame.
+   */
+  if (G.is_rendering && thread_data.rendering_depth == 0 && !BLI_thread_is_main()) {
+    std::scoped_lock lock(device.orphaned_data.mutex_get());
+    device.orphaned_data.move_data(device.orphaned_data_render,
+                                   device.orphaned_data.timeline_ + 1);
+  }
 }
 
-void VKBackend::render_step(bool /*force_resource_release*/) {}
+void VKBackend::render_step(bool force_resource_release)
+{
+  if (force_resource_release) {
+    std::scoped_lock lock(device.orphaned_data.mutex_get());
+    device.orphaned_data.move_data(device.orphaned_data_render,
+                                   device.orphaned_data.timeline_ + 1);
+  }
+}
 
 void VKBackend::capabilities_init(VKDevice &device)
 {
@@ -581,7 +698,7 @@ void VKBackend::capabilities_init(VKDevice &device)
   /* Reset all capabilities from previous context. */
   GCaps = {};
   GCaps.geometry_shader_support = true;
-  GCaps.texture_view_support = true;
+  GCaps.clip_control_support = true;
   GCaps.stencil_export_support = device.supports_extension(
       VK_EXT_SHADER_STENCIL_EXPORT_EXTENSION_NAME);
   GCaps.shader_draw_parameters_support =
