@@ -17,7 +17,7 @@
 #include "IMB_imbuf.hh"
 #include "IMB_imbuf_types.hh"
 
-bool IMB_saveiff(ImBuf *ibuf, const char *filepath, int flags)
+bool IMB_save_image(ImBuf *ibuf, const char *filepath, const int flags)
 {
   errno = 0;
 
@@ -42,7 +42,7 @@ bool IMB_saveiff(ImBuf *ibuf, const char *filepath, int flags)
   if (!(type->flag & IM_FTYPE_FLOAT)) {
     if (ibuf->byte_buffer.data == nullptr && ibuf->float_buffer.data) {
       ibuf->byte_buffer.colorspace = colormanage_colorspace_get_roled(COLOR_ROLE_DEFAULT_BYTE);
-      IMB_rect_from_float(ibuf);
+      IMB_byte_from_float(ibuf);
     }
   }
 
