@@ -28,15 +28,33 @@ struct SpaceType;
 struct uiBlock;
 struct uiLayout;
 struct uiList;
+struct uiListType;
 struct wmDrawBuffer;
 struct wmTimer;
 struct wmTooltipState;
+struct Panel_Runtime;
+#ifdef __cplusplus
+namespace blender::bke {
+struct ARegionRuntime;
+struct FileHandlerType;
+}  // namespace blender::bke
+using ARegionRuntimeHandle = blender::bke::ARegionRuntime;
+using FileHandlerTypeHandle = blender::bke::FileHandlerType;
+#else
+typedef struct ARegionRuntimeHandle ARegionRuntimeHandle;
+typedef struct FileHandlerTypeHandle FileHandlerTypeHandle;
+#endif
 
 /* TODO: Doing this is quite ugly :)
  * Once the top-bar is merged bScreen should be refactored to use ScrAreaMap. */
 #define AREAMAP_FROM_SCREEN(screen) ((ScrAreaMap *)&(screen)->vertbase)
 
 typedef struct bScreen {
+#ifdef __cplusplus
+  /** See #ID_Type comment for why this is here. */
+  static constexpr ID_Type id_type = ID_SCR;
+#endif
+
   ID id;
 
   /* TODO: Should become ScrAreaMap now.
@@ -120,28 +138,23 @@ typedef struct ScrAreaMap {
   ListBase areabase;
 } ScrAreaMap;
 
-typedef struct Panel_Runtime {
-  /* Applied to Panel.ofsx, but saved separately so we can track changes between redraws. */
-  int region_ofsx;
-
-  char _pad[4];
-
+typedef struct LayoutPanelState {
+  struct LayoutPanelState *next, *prev;
+  /** Identifier of the panel. */
+  char *idname;
+  uint8_t flag;
+  char _pad[3];
   /**
-   * Pointer for storing which data the panel corresponds to.
-   * Useful when there can be multiple instances of the same panel type.
-   *
-   * \note A panel and its sub-panels share the same custom data pointer.
-   * This avoids freeing the same pointer twice when panels are removed.
+   * A logical time set from #layout_panel_states_clock when the panel is used by the UI. This is
+   * used to detect the least-recently-used panel states when some panel states should be removed.
    */
-  struct PointerRNA *custom_data_ptr;
+  uint32_t last_used;
+} LayoutPanelState;
 
-  /* Pointer to the panel's block. Useful when changes to panel #uiBlocks
-   * need some context from traversal of the panel "tree". */
-  struct uiBlock *block;
-
-  /* Non-owning pointer. The context is stored in the block. */
-  struct bContextStore *context;
-} Panel_Runtime;
+enum LayoutPanelStateFlag {
+  /** If set, the panel is currently open. Otherwise it is collapsed. */
+  LAYOUT_PANEL_STATE_FLAG_OPEN = (1 << 0),
+};
 
 /** The part from uiBlock that needs saved in file. */
 typedef struct Panel {
@@ -152,8 +165,7 @@ typedef struct Panel {
   /** Runtime for drawing. */
   struct uiLayout *layout;
 
-  /** Defined as UI_MAX_NAME_STR. */
-  char panelname[64];
+  char panelname[/*BKE_ST_MAXNAME*/ 64];
   /** Panel name is identifier for restoring location. */
   char *drawname;
   /** Offset within the region. */
@@ -172,7 +184,21 @@ typedef struct Panel {
   /** Sub panels. */
   ListBase children;
 
-  Panel_Runtime runtime;
+  /**
+   * List of #LayoutPanelState. This stores the open-close-state of layout-panels created with
+   * `layout.panel(...)` in Python. For more information on layout-panels, see
+   * `uiLayout::panel_prop`.
+   */
+  ListBase layout_panel_states;
+  /**
+   * This is increased whenever a layout panel state is used by the UI. This is used to allow for
+   * some garbage collection of panel states when #layout_panel_states becomes large. It works by
+   * removing all least-recently-used panel states up to a certain threshold.
+   */
+  uint32_t layout_panel_states_clock;
+  char _pad2[4];
+
+  struct Panel_Runtime *runtime;
 } Panel;
 
 /**
@@ -269,7 +295,9 @@ typedef struct uiListDyn {
   void *customdata;
 
   /* Filtering data. */
-  /** Items_len length. */
+  /** This bit-field is effectively exposed in Python, and scripts are explicitly allowed to assign
+   * any own meaning to the lower 16 ones.
+   * #items_len length. */
   int *items_filter_flags;
   /** Org_idx -> new_idx, items_len length. */
   int *items_filter_neworder;
@@ -286,8 +314,7 @@ typedef struct uiList { /* some list UI data need to be saved in file */
   /** Runtime. */
   struct uiListType *type;
 
-  /** Defined as UI_MAX_NAME_STR. */
-  char list_id[64];
+  char list_id[/*UI_MAX_NAME_STR*/ 128];
 
   /** How items are laid out in the list. */
   int layout_type;
@@ -299,8 +326,8 @@ typedef struct uiList { /* some list UI data need to be saved in file */
   int list_last_activei;
 
   /* Filtering data. */
-  /** Defined as UI_MAX_NAME_STR. */
-  char filter_byname[64];
+  /** Defined as . */
+  char filter_byname[/*UI_MAX_NAME_STR*/ 128];
   int filter_flag;
   int filter_sort_flag;
 
@@ -311,10 +338,39 @@ typedef struct uiList { /* some list UI data need to be saved in file */
   uiListDyn *dyn_data;
 } uiList;
 
+/** See #uiViewStateLink. */
+typedef struct uiViewState {
+  /**
+   * User set height of the view in unscaled pixels. A value of 0 means no custom height was set
+   * and the default should be used.
+   */
+  int custom_height;
+  /**
+   * Amount of vertical scrolling. View types decide on the unit:
+   * - Tree views: Number of items scrolled out of view (#scroll_offset of 5 means 5 items are
+   *   scrolled out of view).
+   */
+  int scroll_offset;
+} uiViewState;
+
+/**
+ * Persistent storage for some state of views (#ui::AbstractView), for storage in a region. The
+ * view state is matched to the view using the view's idname.
+ *
+ * The actual state is stored in #uiViewState, so views can manage this conveniently without having
+ * to care about the idname and listbase pointers themselves.
+ */
+typedef struct uiViewStateLink {
+  struct uiViewStateLink *next, *prev;
+
+  char idname[/*BKE_ST_MAXNAME*/ 64];
+
+  uiViewState state;
+} uiViewStateLink;
+
 typedef struct TransformOrientation {
   struct TransformOrientation *next, *prev;
-  /** MAX_NAME. */
-  char name[64];
+  char name[/*MAX_NAME*/ 64];
   float mat[3][3];
   char _pad[4];
 } TransformOrientation;
@@ -323,12 +379,21 @@ typedef struct TransformOrientation {
 typedef struct uiPreview {
   struct uiPreview *next, *prev;
 
-  /** Defined as #UI_MAX_NAME_STR. */
-  char preview_id[64];
+  char preview_id[/*BKE_ST_MAXNAME*/ 64];
   short height;
-  char _pad1[6];
+
+  /* Unset on file read. */
+  short tag; /* #uiPreviewTag */
+
+  /** #ID.session_uid of the ID this preview is made for. Unset on file read. */
+  unsigned int id_session_uid;
 } uiPreview;
 
+typedef enum uiPreviewTag {
+  /** Preview needs re-rendering, handled in #ED_preview_draw(). */
+  UI_PREVIEW_TAG_DIRTY = (1 << 0),
+} uiPreviewTag;
+
 typedef struct ScrGlobalAreaData {
   /**
    * Global areas have a non-dynamic size. That means, changing the window size doesn't
@@ -377,15 +442,15 @@ typedef struct ScrArea {
   /** Rect bound by v1 v2 v3 v4. */
   rcti totrct;
 
+  /** eSpace_Type (SPACE_FOO). */
+  char spacetype;
   /**
    * eSpace_Type (SPACE_FOO).
    *
    * Temporarily used while switching area type, otherwise this should be SPACE_EMPTY.
-   * Also, versioning uses it to nicely replace deprecated * editors.
+   * Also, versioning uses it to nicely replace deprecated editors.
    * It's been there for ages, name doesn't fit any more.
    */
-  char spacetype;
-  /** #eSpace_Type (SPACE_FOO). */
   char butspacetype;
   short butspacetype_subtype;
 
@@ -432,24 +497,6 @@ typedef struct ScrArea {
   ScrArea_Runtime runtime;
 } ScrArea;
 
-typedef struct ARegion_Runtime {
-  /** Panel category to use between 'layout' and 'draw'. */
-  const char *category;
-
-  /**
-   * The visible part of the region, use with region overlap not to draw
-   * on top of the overlapping regions.
-   *
-   * Lazy initialize, zero'd when unset, relative to #ARegion.winrct x/y min. */
-  rcti visible_rect;
-
-  /* The offset needed to not overlap with window scroll-bars. Only used by HUD regions for now. */
-  int offset_x, offset_y;
-
-  /** Maps #uiBlock::name to uiBlock for faster lookups. */
-  struct GHash *block_name_map;
-} ARegion_Runtime;
-
 typedef struct ARegion {
   struct ARegion *next, *prev;
 
@@ -457,8 +504,6 @@ typedef struct ARegion {
   View2D v2d;
   /** Coordinates of region. */
   rcti winrct;
-  /** Runtime for partial redraw, same or smaller than winrct. */
-  rcti drawrct;
   /** Size. */
   short winx, winy;
   /**
@@ -466,10 +511,7 @@ typedef struct ARegion {
    * where zero represents no scroll - the first category always shows first at the top.
    */
   int category_scroll;
-  char _pad0[4];
 
-  /** Region is currently visible on screen. */
-  short visible;
   /** Window, header, etc. identifier for drawing. */
   short regiontype;
   /** How it should split. */
@@ -483,20 +525,13 @@ typedef struct ARegion {
    */
   short sizex, sizey;
 
-  /** Private, cached notifier events. */
-  short do_draw;
-  /** Private, cached notifier events. */
-  short do_draw_paintcursor;
   /** Private, set for indicate drawing overlapped. */
   short overlap;
   /** Temporary copy of flag settings for clean full-screen. */
   short flagfullscreen;
 
-  /** Callbacks for this region type. */
-  struct ARegionType *type;
+  char _pad[2];
 
-  /** #uiBlock. */
-  ListBase uiblocks;
   /** Panel. */
   ListBase panels;
   /** Stack of panel categories. */
@@ -505,23 +540,16 @@ typedef struct ARegion {
   ListBase ui_lists;
   /** #uiPreview. */
   ListBase ui_previews;
-  /** #wmEventHandler. */
-  ListBase handlers;
-  /** Panel categories runtime. */
-  ListBase panels_category;
+  /**
+   * Permanent state storage of #ui::AbstractView instances, so hiding regions with views or
+   * loading files remembers the view state.
+   */
+  ListBase view_states; /* #uiViewStateLink */
 
-  /** Gizmo-map of this region. */
-  struct wmGizmoMap *gizmo_map;
-  /** Blend in/out. */
-  struct wmTimer *regiontimer;
-  struct wmDrawBuffer *draw_buffer;
-
-  /** Use this string to draw info. */
-  char *headerstr;
   /** XXX 2.50, need spacedata equivalent? */
   void *regiondata;
 
-  ARegion_Runtime runtime;
+  ARegionRuntimeHandle *runtime;
 } ARegion;
 
 /** #ScrArea.flag */
@@ -550,7 +578,7 @@ enum {
   AREA_FLAG_OFFSCREEN = (1 << 9),
 };
 
-#define AREAGRID 4
+#define AREAGRID 1
 #define AREAMINX 29
 #define HEADER_PADDING_Y 6
 #define HEADERY (20 + HEADER_PADDING_Y)
@@ -620,10 +648,18 @@ enum {
 /** Value (in number of items) we have to go below minimum shown items to enable auto size. */
 #define UI_LIST_AUTO_SIZE_THRESHOLD 1
 
-/* uiList filter flags (dyn_data) */
-/* WARNING! Those values are used by integer RNA too, which does not handle well values > INT_MAX.
- *          So please do not use 32nd bit here. */
+/** uiList filter flags (dyn_data)
+ *
+ * \warning Lower 16 bits are meant for custom use in Python, don't use them here! Only use the
+ *          higher 16 bits.
+ * \warning Those values are used by integer RNA too, which does not handle well values > INT_MAX.
+ *          So please do not use 32nd bit here.
+ */
 enum {
+  /* Don't use (1 << 0) to (1 << 15) here! See warning above. */
+
+  /* Filtering returned #UI_LIST_ITEM_NEVER_SHOW. */
+  UILST_FLT_ITEM_NEVER_SHOW = (1 << 16),
   UILST_FLT_ITEM = 1 << 30, /* This item has passed the filter process successfully. */
 };
 
@@ -776,11 +812,9 @@ enum {
 };
 
 typedef struct AssetShelfSettings {
-  struct AssetShelfSettings *next, *prev;
-
   AssetLibraryReference asset_library_reference;
 
-  ListBase enabled_catalog_paths; /* #LinkData */
+  ListBase enabled_catalog_paths; /* #AssetCatalogPathLink */
   /** If not set (null or empty string), all assets will be displayed ("All" catalog behavior). */
   const char *active_catalog_path;
 
@@ -808,14 +842,16 @@ typedef struct AssetShelf {
 
   /** Identifier that matches the #AssetShelfType.idname this shelf was created with. Used to
    * restore the #AssetShelf.type pointer below on file read. */
-  char idname[64]; /* MAX_NAME */
+  char idname[/*MAX_NAME*/ 64];
   /** Runtime. */
   struct AssetShelfType *type;
 
   AssetShelfSettings settings;
 
+  /** Only for the permanent asset shelf regions, not asset shelves in temporary popups. */
   short preferred_row_count;
-  char _pad[6];
+  short instance_flag;
+  char _pad[4];
 } AssetShelf;
 
 /**
@@ -836,6 +872,8 @@ typedef struct RegionAssetShelf {
   AssetShelf *active_shelf; /* Non-owning. */
 #ifdef __cplusplus
   static RegionAssetShelf *get_from_asset_shelf_region(const ARegion &region);
+  /** Creates the asset shelf region data if necessary, and returns it. */
+  static RegionAssetShelf *ensure_from_asset_shelf_region(ARegion &region);
 #endif
 } RegionAssetShelf;
 
@@ -844,3 +882,20 @@ typedef enum AssetShelfSettings_DisplayFlag {
   ASSETSHELF_SHOW_NAMES = (1 << 0),
 } AssetShelfSettings_DisplayFlag;
 ENUM_OPERATORS(AssetShelfSettings_DisplayFlag, ASSETSHELF_SHOW_NAMES);
+
+/* #AssetShelfSettings.instance_flag */
+typedef enum AssetShelf_InstanceFlag {
+  /**
+   * Remember the last known region visibility state or this shelf, so it can be restored if the
+   * shelf is reactivated. Practically this makes the shelf visibility be remembered per mode.
+   * Continuously updated for the visible region.
+   */
+  ASSETSHELF_REGION_IS_HIDDEN = (1 << 0),
+} AssetShelf_InstanceFlag;
+ENUM_OPERATORS(AssetShelf_InstanceFlag, ASSETSHELF_REGION_IS_HIDDEN);
+
+typedef struct FileHandler {
+  DNA_DEFINE_CXX_METHODS(FileHandler)
+  /** Runtime. */
+  FileHandlerTypeHandle *type;
+} FileHandler;
