@@ -4,6 +4,8 @@
 
 #include "node_function_util.hh"
 
+#include "NOD_geometry_nodes_gizmos.hh"
+
 #include "UI_interface.hh"
 #include "UI_resources.hh"
 
@@ -11,13 +13,13 @@ namespace blender::nodes::node_fn_input_vector_cc {
 
 static void node_declare(NodeDeclarationBuilder &b)
 {
-  b.add_output<decl::Vector>("Vector");
-}
-
-static void node_layout(uiLayout *layout, bContext * /*C*/, PointerRNA *ptr)
-{
-  uiLayout *col = uiLayoutColumn(layout, true);
-  uiItemR(col, ptr, "vector", UI_ITEM_R_EXPAND, "", ICON_NONE);
+  b.add_output<decl::Vector>("Vector").custom_draw([](CustomSocketDrawParams &params) {
+    uiLayout &row = params.layout.row(true);
+    row.column(true).prop(&params.node_ptr, "vector", UI_ITEM_NONE, "", ICON_NONE);
+    if (gizmos::value_node_has_gizmo(params.tree, params.node)) {
+      row.prop(&params.socket_ptr, "pin_gizmo", UI_ITEM_NONE, "", ICON_GIZMO);
+    }
+  });
 }
 
 static void node_build_multi_function(NodeMultiFunctionBuilder &builder)
@@ -30,7 +32,7 @@ static void node_build_multi_function(NodeMultiFunctionBuilder &builder)
 
 static void node_init(bNodeTree * /*tree*/, bNode *node)
 {
-  NodeInputVector *data = MEM_cnew<NodeInputVector>(__func__);
+  NodeInputVector *data = MEM_callocN<NodeInputVector>(__func__);
   node->storage = data;
 }
 
@@ -45,10 +47,9 @@ static void node_register()
   ntype.declare = node_declare;
   ntype.initfunc = node_init;
   blender::bke::node_type_storage(
-      &ntype, "NodeInputVector", node_free_standard_storage, node_copy_standard_storage);
+      ntype, "NodeInputVector", node_free_standard_storage, node_copy_standard_storage);
   ntype.build_multi_function = node_build_multi_function;
-  ntype.draw_buttons = node_layout;
-  blender::bke::node_register_type(&ntype);
+  blender::bke::node_register_type(ntype);
 }
 NOD_REGISTER_NODE(node_register)
 
