@@ -13,16 +13,18 @@
 
 #include "BLI_utildefines.h"
 
-#include "GPU_shader.h"
+#include "GPU_shader.hh"
 #include "intern/gpu_shader_create_info.hh"
 
-#include "../generic/py_capi_utils.h"
-#include "../generic/python_compat.h"
+#include "../generic/py_capi_utils.hh"
+#include "../generic/python_compat.hh" /* IWYU pragma: keep. */
 
-#include "gpu_py_shader.h" /* own include */
+#include "gpu_py_shader.hh" /* own include */
+#include "gpu_py_texture.hh"
 
-//#define USE_PYGPU_SHADER_INFO_IMAGE_METHOD
+#define USE_PYGPU_SHADER_INFO_IMAGE_METHOD
 
+using blender::gpu::shader::DepthWrite;
 using blender::gpu::shader::DualBlend;
 using blender::gpu::shader::Frequency;
 using blender::gpu::shader::ImageType;
@@ -37,10 +39,10 @@ using blender::gpu::shader::Qualifier;
     "      - ``NO_RESTRICT``\n" \
     "      - ``READ``\n" \
     "      - ``WRITE``\n"
-static const struct PyC_FlagSet pygpu_qualifiers[] = {
-    {int(Qualifier::NO_RESTRICT), "NO_RESTRICT"},
-    {int(Qualifier::READ), "READ"},
-    {int(Qualifier::WRITE), "WRITE"},
+static const PyC_FlagSet pygpu_qualifiers[] = {
+    {int(Qualifier::no_restrict), "NO_RESTRICT"},
+    {int(Qualifier::read), "READ"},
+    {int(Qualifier::write), "WRITE"},
     {0, nullptr},
 };
 #endif
@@ -62,21 +64,21 @@ static const struct PyC_FlagSet pygpu_qualifiers[] = {
   "      - ``IVEC4``\n" \
   "      - ``BOOL``\n"
 const PyC_StringEnumItems pygpu_attrtype_items[] = {
-    {int(Type::FLOAT), "FLOAT"},
-    {int(Type::VEC2), "VEC2"},
-    {int(Type::VEC3), "VEC3"},
-    {int(Type::VEC4), "VEC4"},
-    {int(Type::MAT3), "MAT3"},
-    {int(Type::MAT4), "MAT4"},
-    {int(Type::UINT), "UINT"},
-    {int(Type::UVEC2), "UVEC2"},
-    {int(Type::UVEC3), "UVEC3"},
-    {int(Type::UVEC4), "UVEC4"},
-    {int(Type::INT), "INT"},
-    {int(Type::IVEC2), "IVEC2"},
-    {int(Type::IVEC3), "IVEC3"},
-    {int(Type::IVEC4), "IVEC4"},
-    {int(Type::BOOL), "BOOL"},
+    {int(Type::float_t), "FLOAT"},
+    {int(Type::float2_t), "VEC2"},
+    {int(Type::float3_t), "VEC3"},
+    {int(Type::float4_t), "VEC4"},
+    {int(Type::float3x3_t), "MAT3"},
+    {int(Type::float4x4_t), "MAT4"},
+    {int(Type::uint_t), "UINT"},
+    {int(Type::uint2_t), "UVEC2"},
+    {int(Type::uint3_t), "UVEC3"},
+    {int(Type::uint4_t), "UVEC4"},
+    {int(Type::int_t), "INT"},
+    {int(Type::int2_t), "IVEC2"},
+    {int(Type::int3_t), "IVEC3"},
+    {int(Type::int4_t), "IVEC4"},
+    {int(Type::bool_t), "BOOL"},
     {0, nullptr},
 };
 
@@ -114,38 +116,44 @@ const PyC_StringEnumItems pygpu_attrtype_items[] = {
   "      - ``DEPTH_CUBE``\n" \
   "      - ``DEPTH_CUBE_ARRAY``\n"
 static const PyC_StringEnumItems pygpu_imagetype_items[] = {
-    {int(ImageType::FLOAT_BUFFER), "FLOAT_BUFFER"},
-    {int(ImageType::FLOAT_1D), "FLOAT_1D"},
-    {int(ImageType::FLOAT_1D_ARRAY), "FLOAT_1D_ARRAY"},
-    {int(ImageType::FLOAT_2D), "FLOAT_2D"},
-    {int(ImageType::FLOAT_2D_ARRAY), "FLOAT_2D_ARRAY"},
-    {int(ImageType::FLOAT_3D), "FLOAT_3D"},
-    {int(ImageType::FLOAT_CUBE), "FLOAT_CUBE"},
-    {int(ImageType::FLOAT_CUBE_ARRAY), "FLOAT_CUBE_ARRAY"},
-    {int(ImageType::INT_BUFFER), "INT_BUFFER"},
-    {int(ImageType::INT_1D), "INT_1D"},
-    {int(ImageType::INT_1D_ARRAY), "INT_1D_ARRAY"},
-    {int(ImageType::INT_2D), "INT_2D"},
-    {int(ImageType::INT_2D_ARRAY), "INT_2D_ARRAY"},
-    {int(ImageType::INT_3D), "INT_3D"},
-    {int(ImageType::INT_CUBE), "INT_CUBE"},
-    {int(ImageType::INT_CUBE_ARRAY), "INT_CUBE_ARRAY"},
-    {int(ImageType::UINT_BUFFER), "UINT_BUFFER"},
-    {int(ImageType::UINT_1D), "UINT_1D"},
-    {int(ImageType::UINT_1D_ARRAY), "UINT_1D_ARRAY"},
-    {int(ImageType::UINT_2D), "UINT_2D"},
-    {int(ImageType::UINT_2D_ARRAY), "UINT_2D_ARRAY"},
-    {int(ImageType::UINT_3D), "UINT_3D"},
-    {int(ImageType::UINT_CUBE), "UINT_CUBE"},
-    {int(ImageType::UINT_CUBE_ARRAY), "UINT_CUBE_ARRAY"},
-    {int(ImageType::SHADOW_2D), "SHADOW_2D"},
-    {int(ImageType::SHADOW_2D_ARRAY), "SHADOW_2D_ARRAY"},
-    {int(ImageType::SHADOW_CUBE), "SHADOW_CUBE"},
-    {int(ImageType::SHADOW_CUBE_ARRAY), "SHADOW_CUBE_ARRAY"},
-    {int(ImageType::DEPTH_2D), "DEPTH_2D"},
-    {int(ImageType::DEPTH_2D_ARRAY), "DEPTH_2D_ARRAY"},
-    {int(ImageType::DEPTH_CUBE), "DEPTH_CUBE"},
-    {int(ImageType::DEPTH_CUBE_ARRAY), "DEPTH_CUBE_ARRAY"},
+    {int(ImageType::FloatBuffer), "FLOAT_BUFFER"},
+    {int(ImageType::Float1D), "FLOAT_1D"},
+    {int(ImageType::Float1DArray), "FLOAT_1D_ARRAY"},
+    {int(ImageType::Float2D), "FLOAT_2D"},
+    {int(ImageType::Float2DArray), "FLOAT_2D_ARRAY"},
+    {int(ImageType::Float3D), "FLOAT_3D"},
+    {int(ImageType::FloatCube), "FLOAT_CUBE"},
+    {int(ImageType::FloatCubeArray), "FLOAT_CUBE_ARRAY"},
+    {int(ImageType::IntBuffer), "INT_BUFFER"},
+    {int(ImageType::Int1D), "INT_1D"},
+    {int(ImageType::Int1DArray), "INT_1D_ARRAY"},
+    {int(ImageType::Int2D), "INT_2D"},
+    {int(ImageType::Int2DArray), "INT_2D_ARRAY"},
+    {int(ImageType::Int3D), "INT_3D"},
+    {int(ImageType::IntCube), "INT_CUBE"},
+    {int(ImageType::IntCubeArray), "INT_CUBE_ARRAY"},
+    {int(ImageType::AtomicInt2D), "INT_2D_ATOMIC"},
+    {int(ImageType::AtomicInt2DArray), "INT_2D_ARRAY_ATOMIC"},
+    {int(ImageType::AtomicInt3D), "INT_3D_ATOMIC"},
+    {int(ImageType::UintBuffer), "UINT_BUFFER"},
+    {int(ImageType::Uint1D), "UINT_1D"},
+    {int(ImageType::Uint1DArray), "UINT_1D_ARRAY"},
+    {int(ImageType::Uint2D), "UINT_2D"},
+    {int(ImageType::Uint2DArray), "UINT_2D_ARRAY"},
+    {int(ImageType::Uint3D), "UINT_3D"},
+    {int(ImageType::UintCube), "UINT_CUBE"},
+    {int(ImageType::UintCubeArray), "UINT_CUBE_ARRAY"},
+    {int(ImageType::AtomicUint2D), "UINT_2D_ATOMIC"},
+    {int(ImageType::AtomicUint2DArray), "UINT_2D_ARRAY_ATOMIC"},
+    {int(ImageType::AtomicUint3D), "UINT_3D_ATOMIC"},
+    {int(ImageType::Shadow2D), "SHADOW_2D"},
+    {int(ImageType::Shadow2DArray), "SHADOW_2D_ARRAY"},
+    {int(ImageType::ShadowCube), "SHADOW_CUBE"},
+    {int(ImageType::ShadowCubeArray), "SHADOW_CUBE_ARRAY"},
+    {int(ImageType::Depth2D), "DEPTH_2D"},
+    {int(ImageType::Depth2DArray), "DEPTH_2D_ARRAY"},
+    {int(ImageType::DepthCube), "DEPTH_CUBE"},
+    {int(ImageType::DepthCubeArray), "DEPTH_CUBE_ARRAY"},
     {0, nullptr},
 };
 
@@ -156,6 +164,61 @@ static const PyC_StringEnumItems pygpu_dualblend_items[] = {
     {0, nullptr},
 };
 
+static const PyC_StringEnumItems pygpu_depth_write_items[] = {
+    {int(DepthWrite::UNCHANGED), "UNCHANGED"},
+    {int(DepthWrite::ANY), "ANY"},
+    {int(DepthWrite::GREATER), "GREATER"},
+    {int(DepthWrite::LESS), "LESS"},
+    {0, nullptr},
+};
+
+#define PYDOC_TEX_FORMAT_ITEMS \
+  "      - ``RGBA8UI``\n" \
+  "      - ``RGBA8I``\n" \
+  "      - ``RGBA8``\n" \
+  "      - ``RGBA32UI``\n" \
+  "      - ``RGBA32I``\n" \
+  "      - ``RGBA32F``\n" \
+  "      - ``RGBA16UI``\n" \
+  "      - ``RGBA16I``\n" \
+  "      - ``RGBA16F``\n" \
+  "      - ``RGBA16``\n" \
+  "      - ``RG8UI``\n" \
+  "      - ``RG8I``\n" \
+  "      - ``RG8``\n" \
+  "      - ``RG32UI``\n" \
+  "      - ``RG32I``\n" \
+  "      - ``RG32F``\n" \
+  "      - ``RG16UI``\n" \
+  "      - ``RG16I``\n" \
+  "      - ``RG16F``\n" \
+  "      - ``RG16``\n" \
+  "      - ``R8UI``\n" \
+  "      - ``R8I``\n" \
+  "      - ``R8``\n" \
+  "      - ``R32UI``\n" \
+  "      - ``R32I``\n" \
+  "      - ``R32F``\n" \
+  "      - ``R16UI``\n" \
+  "      - ``R16I``\n" \
+  "      - ``R16F``\n" \
+  "      - ``R16``\n" \
+  "      - ``R11F_G11F_B10F``\n" \
+  "      - ``DEPTH32F_STENCIL8``\n" \
+  "      - ``DEPTH24_STENCIL8``\n" \
+  "      - ``SRGB8_A8``\n" \
+  "      - ``RGB16F``\n" \
+  "      - ``SRGB8_A8_DXT1``\n" \
+  "      - ``SRGB8_A8_DXT3``\n" \
+  "      - ``SRGB8_A8_DXT5``\n" \
+  "      - ``RGBA8_DXT1``\n" \
+  "      - ``RGBA8_DXT3``\n" \
+  "      - ``RGBA8_DXT5``\n" \
+  "      - ``DEPTH_COMPONENT32F``\n" \
+  "      - ``DEPTH_COMPONENT24``\n" \
+  "      - ``DEPTH_COMPONENT16``\n"
+extern const PyC_StringEnumItems pygpu_tex_format_items[];
+
 /* -------------------------------------------------------------------- */
 /** \name GPUStageInterfaceInfo Methods
  * \{ */
@@ -179,7 +242,7 @@ static bool pygpu_interface_info_get_args(BPyGPUStageInterfaceInfo *self,
   }
 
 #ifdef USE_GPU_PY_REFERENCES
-  PyList_Append(self->references, (PyObject *)py_name);
+  PyList_Append(self->references, py_name);
 #endif
 
   *r_type = (Type)pygpu_type.value_found;
@@ -187,17 +250,19 @@ static bool pygpu_interface_info_get_args(BPyGPUStageInterfaceInfo *self,
   return true;
 }
 
-PyDoc_STRVAR(pygpu_interface_info_smooth_doc,
-             ".. method:: smooth(type, name)\n"
-             "\n"
-             "   Add an attribute with qualifier of type `smooth` to the interface block.\n"
-             "\n"
-             "   :arg type: One of these types:\n"
-             "\n" PYDOC_TYPE_LIST
-             "\n"
-             "   :type type: str\n"
-             "   :arg name: name of the attribute.\n"
-             "   :type name: str\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_interface_info_smooth_doc,
+    ".. method:: smooth(type, name)\n"
+    "\n"
+    "   Add an attribute with qualifier of type *smooth* to the interface block.\n"
+    "\n"
+    "   :arg type: One of these types:\n"
+    "\n" PYDOC_TYPE_LIST
+    "\n"
+    "   :type type: str\n"
+    "   :arg name: name of the attribute.\n"
+    "   :type name: str\n");
 static PyObject *pygpu_interface_info_smooth(BPyGPUStageInterfaceInfo *self, PyObject *args)
 {
   Type type;
@@ -211,17 +276,19 @@ static PyObject *pygpu_interface_info_smooth(BPyGPUStageInterfaceInfo *self, PyO
   Py_RETURN_NONE;
 }
 
-PyDoc_STRVAR(pygpu_interface_info_flat_doc,
-             ".. method:: flat(type, name)\n"
-             "\n"
-             "   Add an attribute with qualifier of type `flat` to the interface block.\n"
-             "\n"
-             "   :arg type: One of these types:\n"
-             "\n" PYDOC_TYPE_LIST
-             "\n"
-             "   :type type: str\n"
-             "   :arg name: name of the attribute.\n"
-             "   :type name: str\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_interface_info_flat_doc,
+    ".. method:: flat(type, name)\n"
+    "\n"
+    "   Add an attribute with qualifier of type ``flat`` to the interface block.\n"
+    "\n"
+    "   :arg type: One of these types:\n"
+    "\n" PYDOC_TYPE_LIST
+    "\n"
+    "   :type type: str\n"
+    "   :arg name: name of the attribute.\n"
+    "   :type name: str\n");
 static PyObject *pygpu_interface_info_flat(BPyGPUStageInterfaceInfo *self, PyObject *args)
 {
   Type type;
@@ -236,10 +303,11 @@ static PyObject *pygpu_interface_info_flat(BPyGPUStageInterfaceInfo *self, PyObj
 }
 
 PyDoc_STRVAR(
+    /* Wrap. */
     pygpu_interface_info_no_perspective_doc,
     ".. method:: no_perspective(type, name)\n"
     "\n"
-    "   Add an attribute with qualifier of type `no_perspective` to the interface block.\n"
+    "   Add an attribute with qualifier of type ``no_perspective`` to the interface block.\n"
     "\n"
     "   :arg type: One of these types:\n"
     "\n" PYDOC_TYPE_LIST
@@ -280,14 +348,16 @@ static PyMethodDef pygpu_interface_info__tp_methods[] = {
 /** \name GPUStageInterfaceInfo Getters and Setters
  * \{ */
 
-PyDoc_STRVAR(pygpu_interface_info_name_doc,
-             "Name of the interface block.\n"
-             "\n"
-             ":type: str");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_interface_info_name_doc,
+    "Name of the interface block.\n"
+    "\n"
+    ":type: str");
 static PyObject *pygpu_interface_info_name_get(BPyGPUStageInterfaceInfo *self, void * /*closure*/)
 {
   StageInterfaceInfo *interface = reinterpret_cast<StageInterfaceInfo *>(self->interface);
-  return PyUnicode_FromString(interface->name.c_str());
+  return PyC_UnicodeFromStdStr(interface->name);
 }
 
 static PyGetSetDef pygpu_interface_info__tp_getseters[] = {
@@ -364,16 +434,18 @@ static void pygpu_interface_info__tp_dealloc(PyObject *self)
   }
 #endif
 
-  Py_TYPE(self)->tp_free((PyObject *)self);
+  Py_TYPE(self)->tp_free(self);
 }
 
-PyDoc_STRVAR(pygpu_interface_info__tp_doc,
-             ".. class:: GPUStageInterfaceInfo(name)\n"
-             "\n"
-             "   List of varyings between shader stages.\n\n"
-             "\n"
-             "   :arg name: Name of the interface block.\n"
-             "   :type value: str\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_interface_info__tp_doc,
+    ".. class:: GPUStageInterfaceInfo(name)\n"
+    "\n"
+    "   List of varyings between shader stages.\n"
+    "\n"
+    "   :arg name: Name of the interface block.\n"
+    "   :type value: str\n");
 PyTypeObject BPyGPUStageInterfaceInfo_Type = {
     /*ob_base*/ PyVarObject_HEAD_INIT(nullptr, 0)
     /*tp_name*/ "GPUStageInterfaceInfo",
@@ -444,19 +516,21 @@ PyTypeObject BPyGPUStageInterfaceInfo_Type = {
 /** \name GPUShaderCreateInfo Methods
  * \{ */
 
-PyDoc_STRVAR(pygpu_shader_info_vertex_in_doc,
-             ".. method:: vertex_in(slot, type, name)\n"
-             "\n"
-             "   Add a vertex shader input attribute.\n"
-             "\n"
-             "   :arg slot: The attribute index.\n"
-             "   :type slot: int\n"
-             "   :arg type: One of these types:\n"
-             "\n" PYDOC_TYPE_LIST
-             "\n"
-             "   :type type: str\n"
-             "   :arg name: name of the attribute.\n"
-             "   :type name: str\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_vertex_in_doc,
+    ".. method:: vertex_in(slot, type, name)\n"
+    "\n"
+    "   Add a vertex shader input attribute.\n"
+    "\n"
+    "   :arg slot: The attribute index.\n"
+    "   :type slot: int\n"
+    "   :arg type: One of these types:\n"
+    "\n" PYDOC_TYPE_LIST
+    "\n"
+    "   :type type: str\n"
+    "   :arg name: name of the attribute.\n"
+    "   :type name: str\n");
 static PyObject *pygpu_shader_info_vertex_in(BPyGPUShaderCreateInfo *self, PyObject *args)
 {
   int slot;
@@ -477,13 +551,15 @@ static PyObject *pygpu_shader_info_vertex_in(BPyGPUShaderCreateInfo *self, PyObj
   Py_RETURN_NONE;
 }
 
-PyDoc_STRVAR(pygpu_shader_info_vertex_out_doc,
-             ".. method:: vertex_out(interface)\n"
-             "\n"
-             "   Add a vertex shader output interface block.\n"
-             "\n"
-             "   :arg interface: Object describing the block.\n"
-             "   :type interface: :class:`gpu.types.GPUStageInterfaceInfo`\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_vertex_out_doc,
+    ".. method:: vertex_out(interface)\n"
+    "\n"
+    "   Add a vertex shader output interface block.\n"
+    "\n"
+    "   :arg interface: Object describing the block.\n"
+    "   :type interface: :class:`gpu.types.GPUStageInterfaceInfo`\n");
 static PyObject *pygpu_shader_info_vertex_out(BPyGPUShaderCreateInfo *self,
                                               BPyGPUStageInterfaceInfo *o)
 {
@@ -503,21 +579,23 @@ static PyObject *pygpu_shader_info_vertex_out(BPyGPUShaderCreateInfo *self,
   Py_RETURN_NONE;
 }
 
-PyDoc_STRVAR(pygpu_shader_info_fragment_out_doc,
-             ".. method:: fragment_out(slot, type, name, blend='NONE')\n"
-             "\n"
-             "   Specify a fragment output corresponding to a framebuffer target slot.\n"
-             "\n"
-             "   :arg slot: The attribute index.\n"
-             "   :type slot: int\n"
-             "   :arg type: One of these types:\n"
-             "\n" PYDOC_TYPE_LIST
-             "\n"
-             "   :type type: str\n"
-             "   :arg name: Name of the attribute.\n"
-             "   :type name: str\n"
-             "   :arg blend: Dual Source Blending Index. It can be 'NONE', 'SRC_0' or 'SRC_1'.\n"
-             "   :type blend: str\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_fragment_out_doc,
+    ".. method:: fragment_out(slot, type, name, blend='NONE')\n"
+    "\n"
+    "   Specify a fragment output corresponding to a framebuffer target slot.\n"
+    "\n"
+    "   :arg slot: The attribute index.\n"
+    "   :type slot: int\n"
+    "   :arg type: One of these types:\n"
+    "\n" PYDOC_TYPE_LIST
+    "\n"
+    "   :type type: str\n"
+    "   :arg name: Name of the attribute.\n"
+    "   :type name: str\n"
+    "   :arg blend: Dual Source Blending Index. It can be 'NONE', 'SRC_0' or 'SRC_1'.\n"
+    "   :type blend: str\n");
 static PyObject *pygpu_shader_info_fragment_out(BPyGPUShaderCreateInfo *self,
                                                 PyObject *args,
                                                 PyObject *kwds)
@@ -564,15 +642,60 @@ static PyObject *pygpu_shader_info_fragment_out(BPyGPUShaderCreateInfo *self,
 }
 
 PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_depth_write_doc,
+    ".. method:: depth_write(value)\n"
+    "\n"
+    "   Specify a depth write behavior when modifying gl_FragDepth.\n"
+    "\n"
+    "   There is a common optimization for GPUs that relies on an early depth\n"
+    "   test to be run before the fragment shader so that the shader evaluation\n"
+    "   can be skipped if the fragment ends up being discarded because it is occluded.\n"
+    "\n"
+    "   This optimization does not affect the final rendering, and is typically\n"
+    "   possible when the fragment does not change the depth programmatically.\n"
+    "   There are, however a class of operations on the depth in the shader which\n"
+    "   could still be performed while allowing the early depth test to operate.\n"
+    "\n"
+    "   This function alters the behavior of the optimization to allow those operations\n"
+    "   to be performed.\n"
+    "\n"
+    "   :arg value: Depth write value. "
+    "It can be 'UNCHANGED' (default), 'ANY', 'GREATER' or 'LESS'.\n"
+    "      :UNCHANGED: disables depth write in a fragment shader and execution of the"
+    "fragments can be optimized away.\n"
+    "      :ANY: enables depth write in a fragment shader for any fragments\n"
+    "      :GREATER: enables depth write in a fragment shader for depth values that"
+    "are greater than the depth value in the output buffer.\n"
+    "      :LESS: enables depth write in a fragment shader for depth values that"
+    "are less than the depth value in the output buffer.\n"
+    "   :type blend: str\n");
+static PyObject *pygpu_shader_info_depth_write(BPyGPUShaderCreateInfo *self, PyObject *args)
+{
+  PyC_StringEnum depth_write = {pygpu_depth_write_items, int(DepthWrite::UNCHANGED)};
+  if (!PyC_ParseStringEnum(args, &depth_write)) {
+    return nullptr;
+  }
+
+  ShaderCreateInfo *info = reinterpret_cast<ShaderCreateInfo *>(self->info);
+  info->depth_write(DepthWrite(depth_write.value_found));
+
+  Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(
+    /* Wrap. */
     pygpu_shader_info_uniform_buf_doc,
     ".. method:: uniform_buf(slot, type_name, name)\n"
     "\n"
-    "   Specify a uniform variable whose type can be one of those declared in `typedef_source`.\n"
+    "   Specify a uniform variable whose type can be one of those declared in "
+    ":meth:`gpu.types.GPUShaderCreateInfo.typedef_source`.\n"
     "\n"
     "   :arg slot: The uniform variable index.\n"
     "   :type slot: int\n"
-    "   :arg type_name: Name of the data type. It can be a struct type defined in the source "
-    "passed through the :meth:`gpu.types.GPUShaderCreateInfo.typedef_source`.\n"
+    "   :arg type_name: Name of the data type. "
+    "It can be a struct type defined in the source passed through the "
+    ":meth:`gpu.types.GPUShaderCreateInfo.typedef_source`.\n"
     "   :type type_name: str\n"
     "   :arg name: The uniform variable name.\n"
     "   :type name: str\n");
@@ -599,6 +722,7 @@ static PyObject *pygpu_shader_info_uniform_buf(BPyGPUShaderCreateInfo *self, PyO
 
 #ifdef USE_PYGPU_SHADER_INFO_IMAGE_METHOD
 PyDoc_STRVAR(
+    /* Wrap. */
     pygpu_shader_info_image_doc,
     ".. method:: image(slot, format, type, name, qualifiers={'NO_RESTRICT'})\n"
     "\n"
@@ -607,7 +731,7 @@ PyDoc_STRVAR(
     "   :arg slot: The image resource index.\n"
     "   :type slot: int\n"
     "   :arg format: The GPUTexture format that is passed to the shader. Possible values are:\n"
-    "" PYDOC_TEX_FORMAT_ITEMS
+    "\n" PYDOC_TEX_FORMAT_ITEMS
     "   :type format: str\n"
     "   :arg type: The data type describing how the image is to be read in the shader. "
     "Possible values are:\n"
@@ -620,17 +744,17 @@ PyDoc_STRVAR(
     "read or written. Possible values are:\n"
     "" PYDOC_QUALIFIERS
     ""
-    "   :type qualifiers: set\n");
+    "   :type qualifiers: set[str]\n");
 static PyObject *pygpu_shader_info_image(BPyGPUShaderCreateInfo *self,
                                          PyObject *args,
                                          PyObject *kwds)
 {
   int slot;
-  struct PyC_StringEnum pygpu_texformat = {pygpu_textureformat_items};
-  struct PyC_StringEnum pygpu_imagetype = {pygpu_imagetype_items};
+  PyC_StringEnum pygpu_texformat = {pygpu_textureformat_items};
+  PyC_StringEnum pygpu_imagetype = {pygpu_imagetype_items};
   const char *name;
   PyObject *py_qualifiers = nullptr;
-  Qualifier qualifier = Qualifier::NO_RESTRICT;
+  Qualifier qualifier = Qualifier::no_restrict;
 
   static const char *_keywords[] = {"slot", "format", "type", "name", "qualifiers", nullptr};
   static _PyArg_Parser _parser = {
@@ -674,7 +798,7 @@ static PyObject *pygpu_shader_info_image(BPyGPUShaderCreateInfo *self,
   info->image(slot,
               (eGPUTextureFormat)pygpu_texformat.value_found,
               qualifier,
-              (ImageType)pygpu_imagetype.value_found,
+              blender::gpu::shader::ImageReadWriteType(pygpu_imagetype.value_found),
               name);
 
   Py_RETURN_NONE;
@@ -682,6 +806,7 @@ static PyObject *pygpu_shader_info_image(BPyGPUShaderCreateInfo *self,
 #endif
 
 PyDoc_STRVAR(
+    /* Wrap. */
     pygpu_shader_info_sampler_doc,
     ".. method:: sampler(slot, type, name)\n"
     "\n"
@@ -703,7 +828,8 @@ static PyObject *pygpu_shader_info_sampler(BPyGPUShaderCreateInfo *self, PyObjec
   const char *name;
 
   if (!PyArg_ParseTuple(
-          args, "iO&s:sampler", &slot, PyC_ParseStringEnum, &pygpu_samplertype, &name)) {
+          args, "iO&s:sampler", &slot, PyC_ParseStringEnum, &pygpu_samplertype, &name))
+  {
     return nullptr;
   }
 
@@ -720,55 +846,55 @@ static PyObject *pygpu_shader_info_sampler(BPyGPUShaderCreateInfo *self, PyObjec
 static int constant_type_size(Type type)
 {
   switch (type) {
-    case Type::BOOL:
-    case Type::FLOAT:
-    case Type::INT:
-    case Type::UINT:
-    case Type::UCHAR4:
-    case Type::CHAR4:
-    case Type::VEC3_101010I2:
-    case Type::USHORT2:
-    case Type::SHORT2:
+    case Type::bool_t:
+    case Type::float_t:
+    case Type::int_t:
+    case Type::uint_t:
+    case Type::uchar4_t:
+    case Type::char4_t:
+    case Type::float3_10_10_10_2_t:
+    case Type::ushort2_t:
+    case Type::short2_t:
       return 4;
       break;
-    case Type::USHORT3:
-    case Type::SHORT3:
+    case Type::ushort3_t:
+    case Type::short3_t:
       return 6;
       break;
-    case Type::VEC2:
-    case Type::UVEC2:
-    case Type::IVEC2:
-    case Type::USHORT4:
-    case Type::SHORT4:
+    case Type::float2_t:
+    case Type::uint2_t:
+    case Type::int2_t:
+    case Type::ushort4_t:
+    case Type::short4_t:
       return 8;
       break;
-    case Type::VEC3:
-    case Type::UVEC3:
-    case Type::IVEC3:
+    case Type::float3_t:
+    case Type::uint3_t:
+    case Type::int3_t:
       return 12;
       break;
-    case Type::VEC4:
-    case Type::UVEC4:
-    case Type::IVEC4:
+    case Type::float4_t:
+    case Type::uint4_t:
+    case Type::int4_t:
       return 16;
       break;
-    case Type::MAT3:
+    case Type::float3x3_t:
       return 36 + 3 * 4;
-    case Type::MAT4:
+    case Type::float4x4_t:
       return 64;
       break;
-    case Type::UCHAR:
-    case Type::CHAR:
+    case Type::uchar_t:
+    case Type::char_t:
       return 1;
       break;
-    case Type::UCHAR2:
-    case Type::CHAR2:
-    case Type::USHORT:
-    case Type::SHORT:
+    case Type::uchar2_t:
+    case Type::char2_t:
+    case Type::ushort_t:
+    case Type::short_t:
       return 2;
       break;
-    case Type::UCHAR3:
-    case Type::CHAR3:
+    case Type::uchar3_t:
+    case Type::char3_t:
       return 3;
       break;
   }
@@ -804,20 +930,21 @@ static int constants_calc_size(ShaderCreateInfo *info)
   return size_prev + (size_prev % 16);
 }
 
-PyDoc_STRVAR(pygpu_shader_info_push_constant_doc,
-             ".. method:: push_constant(type, name, size=0)\n"
-             "\n"
-             "   Specify a global access constant.\n"
-             "\n"
-             "   :arg type: One of these types:\n"
-             "\n" PYDOC_TYPE_LIST
-             "\n"
-             "   :type type: str\n"
-             "   :arg name: Name of the constant.\n"
-             "   :type name: str\n"
-             "   :arg size: If not zero, indicates that the constant is an array with the "
-             "specified size.\n"
-             "   :type size: uint\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_push_constant_doc,
+    ".. method:: push_constant(type, name, size=0)\n"
+    "\n"
+    "   Specify a global access constant.\n"
+    "\n"
+    "   :arg type: One of these types:\n"
+    "\n" PYDOC_TYPE_LIST
+    "\n"
+    "   :type type: str\n"
+    "   :arg name: Name of the constant.\n"
+    "   :type name: str\n"
+    "   :arg size: If not zero, indicates that the constant is an array with the specified size.\n"
+    "   :type size: int\n");
 static PyObject *pygpu_shader_info_push_constant(BPyGPUShaderCreateInfo *self,
                                                  PyObject *args,
                                                  PyObject *kwds)
@@ -862,6 +989,7 @@ static PyObject *pygpu_shader_info_push_constant(BPyGPUShaderCreateInfo *self,
 }
 
 PyDoc_STRVAR(
+    /* Wrap. */
     pygpu_shader_info_vertex_source_doc,
     ".. method:: vertex_source(source)\n"
     "\n"
@@ -877,7 +1005,7 @@ PyDoc_STRVAR(
     "   :type source: str\n"
     "\n"
     "   .. seealso:: `GLSL Cross Compilation "
-    "<https://wiki.blender.org/wiki/EEVEE_%26_Viewport/GPU_Module/GLSL_Cross_Compilation>`__\n");
+    "<https://developer.blender.org/docs/features/gpu/glsl_cross_compilation/>`__\n");
 static PyObject *pygpu_shader_info_vertex_source(BPyGPUShaderCreateInfo *self, PyObject *o)
 {
   const char *vertex_source = PyUnicode_AsUTF8(o);
@@ -896,13 +1024,60 @@ static PyObject *pygpu_shader_info_vertex_source(BPyGPUShaderCreateInfo *self, P
 #endif
 
   ShaderCreateInfo *info = reinterpret_cast<ShaderCreateInfo *>(self->info);
-  info->vertex_source("common_colormanagement_lib.glsl");
+  info->vertex_source("draw_colormanagement_lib.glsl");
   info->vertex_source_generated = vertex_source;
 
   Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_compute_source_doc,
+    ".. method:: compute_source(source)\n"
+    "\n"
+    "   compute shader source code written in GLSL.\n"
+    "\n"
+    "   Example:\n"
+    "\n"
+    "   .. code-block:: python\n"
+    "\n"
+    "      \"\"\"void main() {\n"
+    "         int2 index = int2(gl_GlobalInvocationID.xy);\n"
+    "         vec4 color = vec4(0.0, 0.0, 0.0, 1.0);\n"
+    "         imageStore(img_output, index, color);\n"
+    "      }\"\"\"\n"
+    "\n"
+    "   :arg source: The compute shader source code.\n"
+    "   :type source: str\n"
+    "\n"
+    "   .. seealso:: `GLSL Cross Compilation "
+    "<https://developer.blender.org/docs/features/gpu/glsl_cross_compilation/>`__\n");
+static PyObject *pygpu_shader_info_compute_source(BPyGPUShaderCreateInfo *self, PyObject *o)
+{
+  const char *compute_source = PyUnicode_AsUTF8(o);
+  if (compute_source == nullptr) {
+    PyErr_Format(PyExc_ValueError, "expected a string, got %s", Py_TYPE(o)->tp_name);
+    return nullptr;
+  }
+
+#ifdef USE_GPU_PY_REFERENCES
+  if (self->compute_source) {
+    Py_DECREF(self->compute_source);
+  }
+
+  self->compute_source = o;
+  Py_INCREF(o);
+#endif
+
+  ShaderCreateInfo *info = reinterpret_cast<ShaderCreateInfo *>(self->info);
+  info->compute_source("draw_colormanagement_lib.glsl");
+  info->compute_source_generated = compute_source;
+
+  Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(
+    /* Wrap. */
     pygpu_shader_info_fragment_source_doc,
     ".. method:: fragment_source(source)\n"
     "\n"
@@ -918,7 +1093,7 @@ PyDoc_STRVAR(
     "   :type source: str\n"
     "\n"
     "   .. seealso:: `GLSL Cross Compilation "
-    "<https://wiki.blender.org/wiki/EEVEE_%26_Viewport/GPU_Module/GLSL_Cross_Compilation>`__\n");
+    "<https://developer.blender.org/docs/features/gpu/glsl_cross_compilation/>`__\n");
 static PyObject *pygpu_shader_info_fragment_source(BPyGPUShaderCreateInfo *self, PyObject *o)
 {
   const char *fragment_source = PyUnicode_AsUTF8(o);
@@ -937,26 +1112,28 @@ static PyObject *pygpu_shader_info_fragment_source(BPyGPUShaderCreateInfo *self,
 #endif
 
   ShaderCreateInfo *info = reinterpret_cast<ShaderCreateInfo *>(self->info);
-  info->fragment_source("common_colormanagement_lib.glsl");
+  info->fragment_source("draw_colormanagement_lib.glsl");
   info->fragment_source_generated = fragment_source;
 
   Py_RETURN_NONE;
 }
 
-PyDoc_STRVAR(pygpu_shader_info_typedef_source_doc,
-             ".. method:: typedef_source(source)\n"
-             "\n"
-             "   Source code included before resource declaration. "
-             "Useful for defining structs used by Uniform Buffers.\n"
-             "\n"
-             "   Example:\n"
-             "\n"
-             ".. code-block:: python\n"
-             "\n"
-             "   \"struct MyType {int foo; float bar;};\"\n"
-             "\n"
-             "   :arg source: The source code defining types.\n"
-             "   :type source: str\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_typedef_source_doc,
+    ".. method:: typedef_source(source)\n"
+    "\n"
+    "   Source code included before resource declaration. "
+    "Useful for defining structs used by Uniform Buffers.\n"
+    "\n"
+    "   Example:\n"
+    "\n"
+    "   .. code-block:: python\n"
+    "\n"
+    "      \"struct MyType {int foo; float bar;};\"\n"
+    "\n"
+    "   :arg source: The source code defining types.\n"
+    "   :type source: str\n");
 static PyObject *pygpu_shader_info_typedef_source(BPyGPUShaderCreateInfo *self, PyObject *o)
 {
   const char *typedef_source = PyUnicode_AsUTF8(o);
@@ -977,7 +1154,7 @@ static PyObject *pygpu_shader_info_typedef_source(BPyGPUShaderCreateInfo *self,
   ShaderCreateInfo *info = reinterpret_cast<ShaderCreateInfo *>(self->info);
 #if 0
   if (info->typedef_sources_.is_empty()) {
-    info->typedef_source("GPU_shader_shared_utils.h");
+    info->typedef_source("GPU_shader_shared_utils.hh");
   }
 #endif
   info->typedef_source_generated = typedef_source;
@@ -985,19 +1162,21 @@ static PyObject *pygpu_shader_info_typedef_source(BPyGPUShaderCreateInfo *self,
   Py_RETURN_NONE;
 }
 
-PyDoc_STRVAR(pygpu_shader_info_define_doc,
-             ".. method:: define(name, value)\n"
-             "\n"
-             "   Add a preprocessing define directive. In GLSL it would be something like:\n"
-             "\n"
-             ".. code-block:: glsl\n"
-             "\n"
-             "   #define name value\n"
-             "\n"
-             "   :arg name: Token name.\n"
-             "   :type name: str\n"
-             "   :arg value: Text that replaces token occurrences.\n"
-             "   :type value: str\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_define_doc,
+    ".. method:: define(name, value)\n"
+    "\n"
+    "   Add a preprocessing define directive. In GLSL it would be something like:\n"
+    "\n"
+    "   .. code-block:: glsl\n"
+    "\n"
+    "      #define name value\n"
+    "\n"
+    "   :arg name: Token name.\n"
+    "   :type name: str\n"
+    "   :arg value: Text that replaces token occurrences.\n"
+    "   :type value: str\n");
 static PyObject *pygpu_shader_info_define(BPyGPUShaderCreateInfo *self, PyObject *args)
 {
   const char *name;
@@ -1025,6 +1204,33 @@ static PyObject *pygpu_shader_info_define(BPyGPUShaderCreateInfo *self, PyObject
   Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info_local_group_size_doc,
+    ".. method:: local_group_size(x, y=1, z=1)\n"
+    "\n"
+    "   Specify the local group size for compute shaders.\n"
+    "\n"
+    "   :arg x: The local group size in the x dimension.\n"
+    "   :type x: int\n"
+    "   :arg y: The local group size in the y dimension. Optional. Defaults to 1.\n"
+    "   :type y: int\n"
+    "   :arg z: The local group size in the z dimension. Optional. Defaults to 1.\n"
+    "   :type z: int\n");
+static PyObject *pygpu_shader_info_local_group_size(BPyGPUShaderCreateInfo *self, PyObject *args)
+{
+  int x = -1, y = -1, z = -1;
+
+  if (!PyArg_ParseTuple(args, "i|ii:local_group_size", &x, &y, &z)) {
+    return nullptr;
+  }
+
+  ShaderCreateInfo *info = reinterpret_cast<ShaderCreateInfo *>(self->info);
+  info->local_group_size(x, y, z);
+
+  Py_RETURN_NONE;
+}
+
 static PyMethodDef pygpu_shader_info__tp_methods[] = {
     {"vertex_in",
      (PyCFunction)pygpu_shader_info_vertex_in,
@@ -1034,6 +1240,10 @@ static PyMethodDef pygpu_shader_info__tp_methods[] = {
      (PyCFunction)pygpu_shader_info_vertex_out,
      METH_O,
      pygpu_shader_info_vertex_out_doc},
+    {"depth_write",
+     (PyCFunction)(void *)pygpu_shader_info_depth_write,
+     METH_O,
+     pygpu_shader_info_depth_write_doc},
     {"fragment_out",
      (PyCFunction)(void *)pygpu_shader_info_fragment_out,
      METH_VARARGS | METH_KEYWORDS,
@@ -1064,11 +1274,19 @@ static PyMethodDef pygpu_shader_info__tp_methods[] = {
      (PyCFunction)pygpu_shader_info_fragment_source,
      METH_O,
      pygpu_shader_info_fragment_source_doc},
+    {"compute_source",
+     (PyCFunction)pygpu_shader_info_compute_source,
+     METH_O,
+     pygpu_shader_info_compute_source_doc},
     {"typedef_source",
      (PyCFunction)pygpu_shader_info_typedef_source,
      METH_O,
      pygpu_shader_info_typedef_source_doc},
     {"define", (PyCFunction)pygpu_shader_info_define, METH_VARARGS, pygpu_shader_info_define_doc},
+    {"local_group_size",
+     (PyCFunction)pygpu_shader_info_local_group_size,
+     METH_VARARGS,
+     pygpu_shader_info_local_group_size_doc},
     {nullptr, nullptr, 0, nullptr},
 };
 
@@ -1098,6 +1316,7 @@ static int pygpu_shader_info__tp_traverse(PyObject *self, visitproc visit, void
   BPyGPUShaderCreateInfo *py_info = reinterpret_cast<BPyGPUShaderCreateInfo *>(self);
   Py_VISIT(py_info->vertex_source);
   Py_VISIT(py_info->fragment_source);
+  Py_VISIT(py_info->compute_source);
   Py_VISIT(py_info->references);
   return 0;
 }
@@ -1107,6 +1326,7 @@ static int pygpu_shader_info__tp_clear(PyObject *self)
   BPyGPUShaderCreateInfo *py_info = reinterpret_cast<BPyGPUShaderCreateInfo *>(self);
   Py_CLEAR(py_info->vertex_source);
   Py_CLEAR(py_info->fragment_source);
+  Py_CLEAR(py_info->compute_source);
   Py_CLEAR(py_info->references);
   return 0;
 }
@@ -1125,17 +1345,21 @@ static void pygpu_shader_info__tp_dealloc(PyObject *self)
     pygpu_shader_info__tp_clear(self);
     Py_XDECREF(py_info->vertex_source);
     Py_XDECREF(py_info->fragment_source);
+    Py_XDECREF(py_info->compute_source);
     Py_XDECREF(py_info->references);
   }
+
 #endif
 
-  Py_TYPE(self)->tp_free((PyObject *)self);
+  Py_TYPE(self)->tp_free(self);
 }
 
-PyDoc_STRVAR(pygpu_shader_info__tp_doc,
-             ".. class:: GPUShaderCreateInfo()\n"
-             "\n"
-             "   Stores and describes types and variables that are used in shader sources.\n");
+PyDoc_STRVAR(
+    /* Wrap. */
+    pygpu_shader_info__tp_doc,
+    ".. class:: GPUShaderCreateInfo()\n"
+    "\n"
+    "   Stores and describes types and variables that are used in shader sources.\n");
 
 PyTypeObject BPyGPUShaderCreateInfo_Type = {
     /*ob_base*/ PyVarObject_HEAD_INIT(nullptr, 0)
@@ -1231,6 +1455,7 @@ PyObject *BPyGPUShaderCreateInfo_CreatePyObject(GPUShaderCreateInfo *info)
   self = (BPyGPUShaderCreateInfo *)_PyObject_GC_New(&BPyGPUShaderCreateInfo_Type);
   self->vertex_source = nullptr;
   self->fragment_source = nullptr;
+  self->compute_source = nullptr;
   self->typedef_source = nullptr;
   self->references = PyList_New(0);
 #else
