@@ -152,8 +152,6 @@ static bool gesture_box_apply(bContext *C, wmOperator *op)
 {
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
 
-  int retval;
-
   if (!gesture_box_apply_rect(op)) {
     return false;
   }
@@ -162,13 +160,13 @@ static bool gesture_box_apply(bContext *C, wmOperator *op)
     gesture_modal_state_to_operator(op, gesture->modal_state);
   }
 
-  retval = op->type->exec(C, op);
+  const wmOperatorStatus retval = op->type->exec(C, op);
   OPERATOR_RETVAL_CHECK(retval);
 
   return (retval & OPERATOR_FINISHED) ? true : false;
 }
 
-int WM_gesture_box_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_box_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
   const ARegion *region = CTX_wm_region(C);
@@ -195,7 +193,7 @@ int WM_gesture_box_invoke(bContext *C, wmOperator *op, const wmEvent *event)
   return OPERATOR_RUNNING_MODAL;
 }
 
-int WM_gesture_box_modal(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_box_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
@@ -262,10 +260,11 @@ int WM_gesture_box_modal(bContext *C, wmOperator *op, const wmEvent *event)
       }
 #endif
 
-#if 0 /* This allows view navigation, keep disabled as it's too unpredictable. */
       default:
+#if 0 /* This allows view navigation, keep disabled as it's too unpredictable. */
         return OPERATOR_PASS_THROUGH;
 #endif
+        break;
     }
   }
 
@@ -291,7 +290,7 @@ void WM_gesture_box_cancel(bContext *C, wmOperator *op)
 
 static void gesture_circle_apply(bContext *C, wmOperator *op);
 
-int WM_gesture_circle_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_circle_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
   const bool wait_for_input = !WM_event_is_mouse_drag_or_press(event) &&
@@ -344,13 +343,12 @@ static void gesture_circle_apply(bContext *C, wmOperator *op)
   }
 
   if (op->type->exec) {
-    int retval;
-    retval = op->type->exec(C, op);
+    const wmOperatorStatus retval = op->type->exec(C, op);
     OPERATOR_RETVAL_CHECK(retval);
   }
 }
 
-int WM_gesture_circle_modal(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_circle_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
@@ -483,7 +481,7 @@ void WM_OT_circle_gesture(wmOperatorType *ot)
  * The operator stores data in the "path" property as a series of screen space positions.
  * \{ */
 
-int WM_gesture_lasso_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_lasso_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
   PropertyRNA *prop;
@@ -504,7 +502,7 @@ int WM_gesture_lasso_invoke(bContext *C, wmOperator *op, const wmEvent *event)
   return OPERATOR_RUNNING_MODAL;
 }
 
-int WM_gesture_lines_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_lines_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
   PropertyRNA *prop;
@@ -527,9 +525,9 @@ int WM_gesture_lines_invoke(bContext *C, wmOperator *op, const wmEvent *event)
   return OPERATOR_RUNNING_MODAL;
 }
 
-static int gesture_lasso_apply(bContext *C, wmOperator *op)
+static wmOperatorStatus gesture_lasso_apply(bContext *C, wmOperator *op)
 {
-  int retval = OPERATOR_FINISHED;
+  wmOperatorStatus retval = OPERATOR_FINISHED;
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
   PointerRNA itemptr;
   float loc[2];
@@ -556,7 +554,7 @@ static int gesture_lasso_apply(bContext *C, wmOperator *op)
   return retval;
 }
 
-int WM_gesture_lasso_modal(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_lasso_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
   const float factor = gesture->use_smooth ? RNA_float_get(op->ptr, "smooth_stroke_factor") : 0.0f;
@@ -631,6 +629,9 @@ int WM_gesture_lasso_modal(bContext *C, wmOperator *op, const wmEvent *event)
         gesture_modal_end(C, op);
         return OPERATOR_CANCELLED;
       }
+      default: {
+        break;
+      }
     }
   }
 
@@ -638,7 +639,7 @@ int WM_gesture_lasso_modal(bContext *C, wmOperator *op, const wmEvent *event)
   return OPERATOR_RUNNING_MODAL;
 }
 
-int WM_gesture_lines_modal(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_lines_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   return WM_gesture_lasso_modal(C, op, event);
 }
@@ -682,7 +683,7 @@ Array<int2> WM_gesture_lasso_path_to_array(bContext * /*C*/, wmOperator *op)
 #if 0
 /* Template to copy from. */
 
-static int gesture_lasso_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus gesture_lasso_exec(bContext *C, wmOperator *op)
 {
   RNA_BEGIN (op->ptr, itemptr, "path") {
     float loc[2];
@@ -725,7 +726,7 @@ void WM_OT_lasso_gesture(wmOperatorType *ot)
  * Like the Lasso Gesture, the data passed onto other operators via the 'path' property is a
  * sequential array of mouse positions.
  * \{ */
-int WM_gesture_polyline_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_polyline_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
   PropertyRNA *prop;
@@ -785,7 +786,9 @@ static bool gesture_polyline_can_apply(const wmGesture &wmGesture, const bool is
   return true;
 }
 
-static int gesture_polyline_apply(bContext *C, wmOperator *op, const bool is_click_submitted)
+static wmOperatorStatus gesture_polyline_apply(bContext *C,
+                                               wmOperator *op,
+                                               const bool is_click_submitted)
 {
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
   BLI_assert(gesture_polyline_can_apply(*gesture, is_click_submitted));
@@ -811,7 +814,7 @@ static int gesture_polyline_apply(bContext *C, wmOperator *op, const bool is_cli
 
   gesture_modal_end(C, op);
 
-  int retval = OPERATOR_FINISHED;
+  wmOperatorStatus retval = OPERATOR_FINISHED;
   if (op->type->exec) {
     retval = op->type->exec(C, op);
     OPERATOR_RETVAL_CHECK(retval);
@@ -820,7 +823,7 @@ static int gesture_polyline_apply(bContext *C, wmOperator *op, const bool is_cli
   return retval;
 }
 
-int WM_gesture_polyline_modal(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_polyline_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
 
@@ -891,6 +894,9 @@ int WM_gesture_polyline_modal(bContext *C, wmOperator *op, const wmEvent *event)
         }
         break;
       }
+      default: {
+        break;
+      }
     }
   }
 
@@ -905,7 +911,7 @@ void WM_gesture_polyline_cancel(bContext *C, wmOperator *op)
 
 /* template to copy from */
 #if 0
-static int gesture_polyline_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus gesture_polyline_exec(bContext *C, wmOperator *op)
 {
   RNA_BEGIN (op->ptr, itemptr, "path") {
     float loc[2];
@@ -988,14 +994,14 @@ static bool gesture_straightline_apply(bContext *C, wmOperator *op)
   RNA_boolean_set(op->ptr, "flip", gesture->use_flip);
 
   if (op->type->exec) {
-    int retval = op->type->exec(C, op);
+    const wmOperatorStatus retval = op->type->exec(C, op);
     OPERATOR_RETVAL_CHECK(retval);
   }
 
   return true;
 }
 
-int WM_gesture_straightline_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_straightline_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
   PropertyRNA *prop;
@@ -1018,7 +1024,9 @@ int WM_gesture_straightline_invoke(bContext *C, wmOperator *op, const wmEvent *e
 
   return OPERATOR_RUNNING_MODAL;
 }
-int WM_gesture_straightline_active_side_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_straightline_active_side_invoke(bContext *C,
+                                                            wmOperator *op,
+                                                            const wmEvent *event)
 {
   WM_gesture_straightline_invoke(C, op, event);
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
@@ -1066,7 +1074,7 @@ static void wm_gesture_straightline_do_angle_snap(rcti *rect, float snap_angle)
   }
 }
 
-int WM_gesture_straightline_modal(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_straightline_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   const Scene *scene = CTX_data_scene(C);
   const ScrArea *area = CTX_wm_area(C);
@@ -1143,6 +1151,9 @@ int WM_gesture_straightline_modal(bContext *C, wmOperator *op, const wmEvent *ev
 
         break;
       }
+      default: {
+        break;
+      }
     }
   }
 
@@ -1150,7 +1161,9 @@ int WM_gesture_straightline_modal(bContext *C, wmOperator *op, const wmEvent *ev
   return OPERATOR_RUNNING_MODAL;
 }
 
-int WM_gesture_straightline_oneshot_modal(bContext *C, wmOperator *op, const wmEvent *event)
+wmOperatorStatus WM_gesture_straightline_oneshot_modal(bContext *C,
+                                                       wmOperator *op,
+                                                       const wmEvent *event)
 {
   const Scene *scene = CTX_data_scene(C);
   const ScrArea *area = CTX_wm_area(C);
@@ -1229,6 +1242,9 @@ int WM_gesture_straightline_oneshot_modal(bContext *C, wmOperator *op, const wmE
 
         break;
       }
+      default: {
+        break;
+      }
     }
   }
 
