@@ -8,7 +8,10 @@
 
 import unittest
 
-from view_layer_common import *
+from view_layer_common import (
+    MoveLayerCollectionTesting,
+    setup_extra_arguments,
+)
 
 
 # ############################################################
